@@ -24,6 +24,7 @@
 #endif
 #include <stdio.h>
 #include <sys/stat.h>
+#include <time.h>
 
 #include "opencdk.h"
 #include "main.h"
@@ -84,8 +85,8 @@ hash_filter (void * opaque, FILE * in, FILE * out)
 static int
 write_header (cipher_filter_s * cfx, CDK_STREAM out)
 {
-  CDK_PACKET pkt;
   cdkPKT_encrypted ed;
+  CDK_PACKET pkt;
   CDK_DEK dek = cfx->dek;
   byte temp[18];
   size_t blocksize = 0;
@@ -140,10 +141,8 @@ write_header (cipher_filter_s * cfx, CDK_STREAM out)
   rc = gcry_cipher_setkey (cfx->cipher_hd, dek->key, dek->keylen);
   if (rc)
     {
-      if (rc == GCRYERR_WEAK_KEY)
-        rc = CDK_Weak_Key;
-      else
-        rc = CDK_Gcry_Error;
+      rc = rc == GCRYERR_WEAK_KEY? CDK_Weak_Key : CDK_Gcry_Error;
+      gcry_cipher_close (cfx->cipher_hd);
       gcry_md_close (cfx->mdc_hash);
       return rc;
     }
@@ -162,9 +161,10 @@ cipher_encode2 (void * opaque, FILE * in, FILE * out)
 {
   struct stat statbuf;
   cipher_filter_s * cfx = opaque;
-  byte buf[4096], pktdata[22];
+  byte buf[8192], pktdata[22];
   int algo = GCRY_MD_SHA1, dlen = gcry_md_get_algo_dlen (algo);
   int rc = 0, nread, len;
+  clock_t start, end;
 
   if (!cfx || !in || !out)
     return CDK_Inv_Value;
@@ -172,7 +172,7 @@ cipher_encode2 (void * opaque, FILE * in, FILE * out)
   if (fstat (fileno (in), &statbuf))
     return CDK_File_Error;
   len = statbuf.st_size;
-  
+  start = clock ();
   while (!feof (in))
     {
       nread = fread (buf, 1, sizeof buf -1, in);
@@ -209,6 +209,8 @@ cipher_encode2 (void * opaque, FILE * in, FILE * out)
     }
   gcry_cipher_close (cfx->cipher_hd);
   cfx->cipher_hd = NULL;
+  end = clock ();
+  /*printf ("cipher encode: elapsed time %d\n", start, end, (end - start));*/
   return rc;
 }
 
@@ -243,10 +245,8 @@ read_header (cipher_filter_s * cfx, FILE * in)
   rc = gcry_cipher_setkey (cfx->cipher_hd, dek->key, dek->keylen);
   if (rc)
     {
-      if (rc == GCRYERR_WEAK_KEY)
-        rc = CDK_Weak_Key;
-      else
-        rc = CDK_Gcry_Error;
+      rc = rc == GCRYERR_WEAK_KEY ? CDK_Weak_Key : CDK_Gcry_Error;
+      gcry_cipher_close (cfx->cipher_hd);
       return rc;
     }
   gcry_cipher_setiv (cfx->cipher_hd, NULL, 0);
@@ -260,10 +260,12 @@ read_header (cipher_filter_s * cfx, FILE * in)
   gcry_cipher_sync (cfx->cipher_hd);
   if (temp[nprefix - 2] != temp[nprefix]
       || temp[nprefix - 1] != temp[nprefix + 1])
-    return CDK_Chksum_Error;
+    rc = CDK_Chksum_Error;
   if (cfx->mdc_hash)
     gcry_md_write (cfx->mdc_hash, temp, nprefix + 2);
-  return 0;
+  if (cfx->blkmode.on)
+    cfx->blkmode.size -= (nprefix + 2);
+  return rc;
 }
 
 
@@ -290,15 +292,18 @@ static int
 cipher_decode2 (void * opaque, FILE * in, FILE * out)
 {
   cipher_filter_s * cfx = opaque;
-  byte buf[4096];
-  int rc = 0, nread;
+  byte buf[8192];
+  int rc = 0, nread, nreq;
 
   if (!cfx || !in || !out)
     return CDK_Inv_Value;
 
   while (!feof (in))
     {
-      nread = fread (buf, 1, sizeof buf-1, in);
+      _cdk_log_debug ("partial on=%d size=%lu\n",
+                      cfx->blkmode.on, cfx->blkmode.size);
+      nreq = cfx->blkmode.on? cfx->blkmode.size: sizeof buf-1;
+      nread = fread (buf, 1, nreq, in);
       if (!nread)
         break;
       rc = gcry_cipher_decrypt (cfx->cipher_hd, buf, nread, NULL, 0);
@@ -312,6 +317,12 @@ cipher_decode2 (void * opaque, FILE * in, FILE * out)
       else if (cfx->mdc_hash)
         gcry_md_write (cfx->mdc_hash, buf, nread);
       fwrite (buf, 1, nread, out);
+      if (cfx->blkmode.on)
+        {
+          cfx->blkmode.size = _cdk_pkt_read_len (in, &cfx->blkmode.on);
+          if (cfx->blkmode.size == (size_t)EOF)
+            return CDK_Inv_Packet;
+        }
     }
   gcry_cipher_close (cfx->cipher_hd);
   cfx->cipher_hd = NULL;
