@@ -23,6 +23,7 @@
 # include <config.h>
 #endif
 #include <stdio.h>
+#include <time.h>
 #ifdef HAVE_LIBZ
 # include <zlib.h>
 #endif
@@ -96,6 +97,28 @@ decompress_data (compress_filter_s * zfx, z_stream * zs,
 }
 
 
+/* in some cases it might be possible that the packet header was not
+   readed, so we do it here. but we assume the the algo was already set. */
+static void
+skip_pktheader (FILE * in)
+{
+  int ctb, pkttype, pos;
+
+  pos = ftell (in);
+  ctb = fgetc (in);
+  if (ctb & 0x80)
+    {
+      pkttype = ctb & 0x40 ? (ctb & 0x3f) : ((ctb >> 2) & 0xf);
+      if (pkttype == CDK_PKT_COMPRESSED)
+        {
+          fgetc (in);
+          return;
+        }
+    }
+  fseek (in, pos, SEEK_SET);
+}
+
+  
 int
 compress_decode (void * opaque, FILE * in, FILE * out)
 {
@@ -104,7 +127,7 @@ compress_decode (void * opaque, FILE * in, FILE * out)
   int zrc, rc = 0;
   size_t nbytes;
 
-  _cdk_log_debug ("compress filter: decode\n");
+  _cdk_log_debug ("compress filter: decode (algo=%d)\n", zfx->algo);
   
   if (!zfx || !in || !out)
     return CDK_Inv_Value;
@@ -125,6 +148,7 @@ compress_decode (void * opaque, FILE * in, FILE * out)
   memset (zfx->inbuf, 0, sizeof zfx->inbuf);
   zs->avail_in = 0;  
 
+  skip_pktheader (in);
   while (rc != -1)
     {
       zs->next_out = zfx->outbuf;
@@ -148,6 +172,7 @@ compress_encode (void * opaque, FILE * in, FILE * out)
   cdkPKT_compressed cd;
   CDK_PACKET pkt;
   CDK_STREAM so;
+  clock_t start, end;
   int rc, zrc, nread;
 
   _cdk_log_debug ("compress filter: encode\n");
@@ -187,6 +212,7 @@ compress_encode (void * opaque, FILE * in, FILE * out)
   zfx->outbufsize = 8192;
   memset (zfx->outbuf, 0, sizeof zfx->outbuf);
 
+  start = clock ();
   while (!feof (in))
     {
       nread = fread (zfx->outbuf, 1, zfx->outbufsize, in);
@@ -206,6 +232,8 @@ compress_encode (void * opaque, FILE * in, FILE * out)
       if (zrc != Z_STREAM_END)
         rc = CDK_Zlib_Error;
     }
+  end = clock ();
+  /*printf ("compress encode: elapsed time %d\n", end - start);*/
   deflateEnd (zs);
   cdk_free (zs);
   return rc;
