@@ -70,8 +70,13 @@ struct cdk_handle_s
 
   CDK_DEK dek;
   CDK_S2K_HD s2k;
+  struct 
+  {
+    CDK_KEYDB_HD sec;
+    CDK_KEYDB_HD pub;
+  } db;
   void (*callback) (void * opaque, int type, const char * s);
-  void *callback_value;
+  void * callback_value;
 };
 
 struct cdk_prefitem_s
@@ -95,22 +100,39 @@ struct cdk_mpi_s
   unsigned char data[1];  
 };
 
+struct key_table_s 
+{
+  struct key_table_s * next;
+  unsigned long offset;
+  CDK_DBSEARCH desc;
+};
+
+struct key_idx_s
+{
+  unsigned long offset;
+  unsigned long keyid[2];
+  unsigned char fpr[20]; 
+};
+  
 struct cdk_keydb_handle_s
 {
-  CDK_STREAM buf; /* NULL if the name item is valid */
-  u32 old_offset;
-  u32 offset;
-  char * name;
   int type;
-  int secret;
-  int used;  
+  CDK_STREAM buf; /* NULL if the name item is valid */
+  CDK_STREAM idx;
+  char * name;
+  char * idx_name;
+  struct key_table_s * cache;
+  int ncache;
+  unsigned int secret:1;
+  unsigned int isopen:1;
+  unsigned int no_cache:1;
 };
 
 struct cdk_keydb_search_s
 {
   union
   {
-    const char * pattern;
+    char * pattern;
     unsigned long keyid[2];
     unsigned char fpr[20];     
   }
