@@ -210,7 +210,7 @@ cdk_stream_encrypt (CDK_HD hd, CDK_STRLIST remusr,
   if (!remusr)
     return sym_stream_encrypt (hd, inp, out);
 
-  rc = cdk_pklist_build (remusr, &pkl, GCRY_PK_USAGE_ENCR);
+  rc = cdk_pklist_build (hd->db.pub, remusr, &pkl, GCRY_PK_USAGE_ENCR);
   if (rc)
     return rc;
 
@@ -320,10 +320,8 @@ is_openpgp_ext (const char * file)
 {
   int yes = 0;
 
-  if (strstr (file, ".asc")
-      || strstr (file, ".gpg")
-      || strstr (file, ".pgp")
-      || strstr (file, ".sig"))
+  if (strstr (file, ".asc") || strstr (file, ".sig")
+      || strstr (file, ".gpg") || strstr (file, ".pgp"))
     yes = 1;
 
   return yes;
@@ -364,7 +362,7 @@ static int
 handle_symkey_enc (CTX c, CDK_PACKET * pkt)
 {
   char * pw;
-  cdkPKT_symkey_enc *key;
+  cdkPKT_symkey_enc * key;
   int rc = 0;
 
   assert (pkt->pkttype == CDK_PKT_SYMKEY_ENC);
@@ -404,7 +402,7 @@ handle_pubkey_enc (CTX c, CDK_HD hd, CDK_PACKET * pkt)
   if (c->dek)
     return 0; /* we already decrypted the session key */
 
-  snode = _cdk_keydb_get_skblock (enc->keyid);
+  snode = _cdk_keydb_get_skblock (hd->db.sec, enc->keyid);
   if (!snode)
     return 0;
   sk = _cdk_keydb_find_bykeyid (snode, enc->keyid);
@@ -447,14 +445,13 @@ rfc1991_get_sesskey (CDK_DEK * r_dek)
 
   
 static int
-handle_encrypted (CTX c, CDK_PACKET *pkt, int use_mdc)
+handle_encrypted (CTX c, CDK_PACKET * pkt, int use_mdc)
 {
   cdkPKT_encrypted * enc;
   int pgp2_compat = _cdk_is_idea_available ();
-  int rc = 0;
+  int rc = 0, pkttype = pkt->pkttype;
 
-  assert (pkt->pkttype == CDK_PKT_ENCRYPTED
-          || pkt->pkttype == CDK_PKT_ENCRYPTED_MDC);
+  assert (pkttype == CDK_PKT_ENCRYPTED || pkttype == CDK_PKT_ENCRYPTED_MDC);
 
   if (!c->dek)
     {
@@ -504,6 +501,9 @@ static int
 handle_onepass_sig (CTX c, CDK_PACKET * pkt)
 {
   assert (pkt->pkttype == CDK_PKT_ONEPASS_SIG);
+
+  if (c->sig.md)
+    return 0; /* already open */               
   
   c->sig.digest_algo = pkt->pkt.onepass_sig->digest_algo;
   c->sig.md = gcry_md_open (c->sig.digest_algo, 0);
@@ -569,7 +569,7 @@ handle_signature (CDK_HD hd, CTX c, CDK_PACKET * pkt)
   res->pubkey_algo = sig->pubkey_algo;
   res->digest_algo = sig->digest_algo;
   
-  rc = cdk_keydb_get_pk (NULL, keyid, &c->sig.pk);
+  rc = cdk_keydb_get_pk (hd->db.pub, keyid, &c->sig.pk);
   if (rc)
     {
       res->sig_status = CDK_SIGSTAT_NOKEY;
@@ -590,8 +590,6 @@ free_mainproc (CTX c)
 {
   if (c)
     {
-      if (c->dek)
-        cdk_stream_close (c->s);
       cdk_kbnode_release (c->node);
       if (c->sig.present)
         {
@@ -788,7 +786,7 @@ _cdk_proc_signature_packets (CDK_HD hd, GCRY_MD_HD md,
  * the signature (when present) and more.
  **/
 int
-cdk_file_decrypt (CDK_HD hd, const char *file, const char *output)
+cdk_file_decrypt (CDK_HD hd, const char * file, const char * output)
 {
   CDK_STREAM inp = NULL, out = NULL;
   int rc;
@@ -800,7 +798,11 @@ cdk_file_decrypt (CDK_HD hd, const char *file, const char *output)
   if (!rc)
     rc = cdk_stream_new (output, &out);
   if (!rc)
-    rc = proc_encryption_packets (hd, inp, out);
+    {
+      if (cdk_armor_filter_use (inp))
+        cdk_stream_set_armor_flag (inp, 0);
+      rc = proc_encryption_packets (hd, inp, out);
+    }
 
   cdk_stream_close (inp);
   cdk_stream_close (out);
