@@ -33,8 +33,12 @@ typedef struct
   GCRY_MD_HD mdc_hash;
   int mdc_method;
   CDK_DEK dek;
-  unsigned char enchash[20];
   unsigned long datalen;
+  struct 
+  {
+    int on;
+    size_t size;
+  } blkmode;
   CDK_STREAM s;
 } cipher_filter_s;
 
@@ -42,7 +46,6 @@ typedef struct
 {
   int digest_algo;
   GCRY_MD_HD md;
-  GCRY_MD_HD md2;  
 } md_filter_s;
 
 typedef struct
