@@ -443,6 +443,12 @@ cdk_kbnode_write_to_mem (CDK_KBNODE node, byte * buf, size_t * r_nbytes)
     {
       cdk_stream_seek (s, 0);
       len = cdk_stream_get_length (s);
+      if (!buf)
+        {
+          *r_nbytes = len; /* only return the length of the buffer */
+          cdk_stream_close (s);
+          return 0;
+        }
       if (*r_nbytes < len)
         rc = CDK_Too_Short;
       if (!rc)
