@@ -34,138 +34,179 @@
 #include "stream.h"
 
 
-#define KEYDB_ENTRIES 4
-static struct cdk_keydb_handle_s keydb_table[KEYDB_ENTRIES];
+#define KEYDB_CACHE_ENTRIES 8
 
-int
-cdk_keydb_add_resource (const char *name, int is_secret)
+typedef struct key_table_s * KEY_TABLE;
+typedef struct key_idx_s * KEY_IDX;
+
+
+static void keydb_cache_free (KEY_TABLE cache);
+static int keydb_search_copy (CDK_DBSEARCH * r_dst, CDK_DBSEARCH src);
+
+     
+static char *
+keydb_idx_mkname (const char * file)
 {
-  int i;
+  char * fname;
+  fname = cdk_calloc (1, strlen (file) + 5);
+  strcpy (fname, file);
+  strcat (fname, ".idx");
+  return fname;
+}
 
-  if (!name)
+
+/* This functions builds an index of the keyring into a separate file
+   with the name keyring.ext.idx. It contains the offset of all public-
+   and public subkeys. The format of the file is:
+   --------
+    4 octets offset of the packet
+    8 octets keyid
+   20 octets fingerprint
+   --------
+   We store the keyid and the fingerprint due to the fact we can't get
+   the keyid from a v3 fingerprint directly.
+*/
+static int
+keydb_idx_build (const char * file)
+{
+  CDK_PACKET pkt;
+  CDK_STREAM inp, out;
+  byte buf[8], fpr[20];
+  u32 keyid[2];
+  char * fname;
+  long pos, pkttype;
+  int rc;
+
+  if (!file)
     return CDK_Inv_Value;
-  for (i = 0; i < KEYDB_ENTRIES; i++)
+  
+  fname = keydb_idx_mkname (file);
+  
+  rc = cdk_stream_open (file, &inp);
+  if (rc)
+    return rc;
+  rc = cdk_stream_new (fname, &out);
+  if (rc)
     {
-      if (!keydb_table[i].used)
-	{
-	  keydb_table[i].name = cdk_calloc (1, strlen (name) + 1);
-	  if (!keydb_table[i].name)
-	    return CDK_Out_Of_Core;
-	  strcpy (keydb_table[i].name, name);
-	  keydb_table[i].secret = is_secret;
-	  keydb_table[i].used = 1;
-	  keydb_table[i].type = CDK_DBTYPE_KEYRING;
-	  keydb_table[i].offset = 0;
-	  keydb_table[i].old_offset = 0;
-	  break;
-	}
+      cdk_stream_close (inp);
+      return rc;
     }
 
-  return 0;
-}
-
-
-int
-cdk_keydb_remove_resource (int id)
-{
-  int i;
-
-  for (i = 0; i < KEYDB_ENTRIES; i++)
+  while (!cdk_stream_eof (inp))
     {
-      if (keydb_table[i].used && id == i)
-	{
-	  cdk_free (keydb_table[i].name);
-	  keydb_table[i].name = NULL;
-	  keydb_table[i].offset = 0;
-	  keydb_table[i].old_offset = 0;
-	  keydb_table[i].secret = 0;
-	  keydb_table[i].used = 0;
-	  break;
-	}
-    }
-  return 0;
-}
-
-
-const char *
-cdk_keydb_get_name (int id)
-{
-  int i;
-
-  if (id == -1)
-    {
-      for (i = 0; i < KEYDB_ENTRIES; i++)
-	{
-	  if (keydb_table[i].used)
-	    return keydb_table[i].name;
-	}
-      return NULL;
-    }
-  for (i = 0; i < KEYDB_ENTRIES; i++)
-    {
-      if (keydb_table[i].used && id == i)
-	return keydb_table[i].name;
-    }
-  return NULL;
-}
-
-
-int
-cdk_keydb_is_secret (int id)
-{
-  int i;
-
-  for (i = 0; i < KEYDB_ENTRIES; i++)
-    {
-      if (keydb_table[i].used && id == i)
-	return keydb_table[i].secret;
-    }
-
-  return 0;
-}
-
-
-int
-cdk_keydb_find_idx (int is_secret, int *ret_pos)
-{
-  int i;
-
-  if (!ret_pos)
-    return CDK_Inv_Value;
-
-  *ret_pos = -1;
-  for (i = 0; i < KEYDB_ENTRIES; i++)
-    {
-      if (!keydb_table[i].used)
-	continue;
-      if (is_secret == cdk_keydb_is_secret (i))
-	{
-	  *ret_pos = i;
-	  return 0;
-	}
-    }
-  return CDK_General_Error;
-}
-
-
-CDK_KEYDB_HD
-cdk_keydb_get_ctx (int is_secret, int id)
-{
-  int idx = 0;
-
-  if (id == -1)
-    {
-      int rc = cdk_keydb_find_idx (is_secret, &idx);
+      pos = cdk_stream_tell (inp);
+      rc = cdk_pkt_parse (inp, &pkt);
       if (rc)
-	return NULL;
-      return &keydb_table[idx];
+        break;
+      pkttype = pkt.pkttype;
+      if (pkttype == CDK_PKT_PUBLIC_KEY || pkttype == CDK_PKT_PUBLIC_SUBKEY)
+        {
+          _cdk_u32tobuf (pos, buf);
+          cdk_stream_write (out, buf, 4);
+          cdk_pk_get_keyid (pkt.pkt.public_key, keyid);
+          _cdk_u32tobuf (keyid[0], buf);
+          _cdk_u32tobuf (keyid[1], buf + 4);
+          cdk_stream_write (out, buf, 8);
+          cdk_pk_get_fingerprint (pkt.pkt.public_key, fpr);
+          cdk_stream_write (out, fpr, 20);
+        }
+      cdk_pkt_free (&pkt);
     }
-  if (id >= 0 && id < KEYDB_ENTRIES)
+  cdk_stream_close (inp);
+  cdk_stream_close (out);
+  cdk_free (fname);
+  return rc;
+}
+
+
+static int
+keydb_idx_rebuild (CDK_KEYDB_HD hd)
+{
+  int rc;
+  
+  if (!hd || !hd->name)
+    return CDK_Inv_Value;
+  if (hd->secret)
+    return 0;
+  
+  cdk_stream_close (hd->idx);
+  if (!hd->idx_name)
+      hd->idx_name = keydb_idx_mkname (hd->name);
+  rc = keydb_idx_build (hd->name);
+  if (!rc)
+    rc = cdk_stream_open (hd->idx_name, &hd->idx);
+  return rc;
+}
+
+
+static int
+keydb_idx_parse (CDK_STREAM inp, KEY_IDX * r_idx)
+{
+  KEY_IDX idx;
+  byte buf[4];
+  int i;
+
+  idx = cdk_calloc (1, sizeof * idx);
+  if (!idx)
+    return CDK_Out_Of_Core;
+
+  while (!cdk_stream_eof (inp))
     {
-      if (keydb_table[id].used && keydb_table[id].secret == is_secret)
-	return &keydb_table[id];
+      i = cdk_stream_read (inp, buf, 4);
+      if (i == CDK_EOF)
+        break;
+      idx->offset = buf[0] << 24 | buf[1] << 16 | buf[2] << 8 | buf[3];
+      cdk_stream_read (inp, buf, 4);
+      idx->keyid[0] = buf[0] << 24 | buf[1] << 16 | buf[2] << 8 | buf[3];
+      cdk_stream_read (inp, buf, 4);
+      idx->keyid[1] = buf[0] << 24 | buf[1] << 16 | buf[2] << 8 | buf[3];
+      cdk_stream_read (inp, idx->fpr, 20);
+      #if 0
+      _cdk_log_debug ("%08lX%08lX %06lu ",
+                      idx->keyid[0], idx->keyid[1], idx->offset);
+      for (i = 0; i < 20; i++)
+        _cdk_log_debug ("%02X", idx->fpr[i]);
+      _cdk_log_debug ("\n");
+      #endif
+      break; 
     }
-  return NULL;
+  if (r_idx)
+    *r_idx = idx;
+  return cdk_stream_eof (inp)? CDK_EOF : 0;  
+}
+
+
+static int
+keydb_idx_search (CDK_STREAM inp, u32 * keyid, const byte * fpr, u32 * r_off)
+{
+  KEY_IDX idx;
+
+  if (keyid && fpr)
+    return CDK_Inv_Mode;
+  if (!keyid && !fpr)
+    return CDK_Inv_Value;
+  if (!r_off)
+    return CDK_Inv_Value;
+
+  *r_off = 0;
+  cdk_stream_seek (inp, 0);
+  while (keydb_idx_parse (inp, &idx) != CDK_EOF)
+    {
+      if (keyid && idx->keyid[0] == keyid[0] && idx->keyid[1] == keyid[1])
+        {
+          *r_off = idx->offset;
+          break; 
+        }
+      else if (fpr && !memcmp (idx->fpr, fpr, 20))
+        {
+          *r_off = idx->offset;
+          break; 
+        }
+      cdk_free (idx);
+      idx = NULL; 
+    }
+  cdk_free (idx);
+  return *r_off? 0 : CDK_EOF;
 }
 
 
@@ -173,30 +214,28 @@ int
 cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, void * data, size_t count)
 {
   CDK_KEYDB_HD hd;
-  char * name;
 
   hd = cdk_calloc (1, sizeof *hd);
   if (!hd)
     return CDK_Out_Of_Core;
   
-  hd->used = 1;
   switch (type)
     {
-    case CDK_DBTYPE_ARMORED:
-    case CDK_DBTYPE_KEYRING:
-      name = data;
-      name[count] = '\0';
-      hd->name = strdup (name);
+    case CDK_DBTYPE_PK_KEYRING:
+    case CDK_DBTYPE_SK_KEYRING:
+      hd->name = cdk_calloc (1, count + 1);
       if (!hd->name)
         {
           cdk_free (hd);
           return CDK_Out_Of_Core;
         }
+      memcpy (hd->name, data, count);
+      hd->name[count] = '\0';
       hd->type = type;
       break;
       
     case CDK_DBTYPE_DATA:
-      hd->buf = cdk_stream_tmp_from_mem (data, count);      
+      hd->buf = cdk_stream_tmp_from_mem (data, count);
       if (!hd->buf)
         {
           cdk_free (hd);
@@ -205,45 +244,97 @@ cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, void * data, size_t count)
       hd->type = CDK_DBTYPE_DATA;
       break;
     }
+  if (type == CDK_DBTYPE_SK_KEYRING)
+    hd->secret = 1;
   if (r_hd)
     *r_hd = hd;
   return 0;
 }
 
 
-int
-cdk_keydb_open (CDK_KEYDB_HD kdb, CDK_STREAM * ret_kr)
+void
+cdk_keydb_free (CDK_KEYDB_HD hd)
 {
-  int rc = 0;
-  CDK_STREAM kr = NULL;
+  if (hd)
+    {
+      if (hd->isopen && hd->name)
+        {
+          cdk_free (hd->name);
+          cdk_stream_close (hd->buf);
+          hd->buf = NULL;
+        }
+      if (!hd->secret)
+        {
+          cdk_stream_close (hd->idx);
+          hd->idx = NULL;
+        }
+      hd->no_cache = 0;
+      hd->secret = 0;
+      hd->isopen = 0;
+      cdk_free (hd);
+    }
+}
 
-  if (!kdb)
+
+int
+cdk_keydb_open (CDK_KEYDB_HD hd, CDK_STREAM * ret_kr)
+{
+  int rc = 0, ec, type;
+
+  if (!hd)
     return CDK_Inv_Value;
 
-  if (kdb->type == CDK_DBTYPE_DATA && kdb->buf)
+  type = hd->type;
+  if (type == CDK_DBTYPE_DATA)
     {
-      cdk_stream_seek (kdb->buf, 0);
-      kr = kdb->buf;
+      if (hd->buf)
+        cdk_stream_seek (hd->buf, 0);
     }
-  else if (kdb->name && (kdb->type == CDK_DBTYPE_KEYRING
-                         || kdb->type == CDK_DBTYPE_ARMORED))
+  else if (type == CDK_DBTYPE_PK_KEYRING || type == CDK_DBTYPE_SK_KEYRING)
     {
-      rc = cdk_stream_open (kdb->name, &kr);
-      if (rc)
-	goto leave;
-      if (kdb->type == CDK_DBTYPE_ARMORED)
-        cdk_stream_set_armor_flag (kr, 0);
+      if (!hd->isopen && hd->name)
+        {
+          rc = cdk_stream_open (hd->name, &hd->buf);
+          if (rc)
+            goto leave;
+          if (cdk_armor_filter_use (hd->buf))
+            cdk_stream_set_armor_flag (hd->buf, 0);
+          hd->isopen = 1;
+          cdk_free (hd->idx_name);
+          hd->idx_name = keydb_idx_mkname (hd->name);
+          ec = cdk_stream_open (hd->idx_name, &hd->idx);
+          if (ec && !hd->secret)
+            {
+              rc = keydb_idx_build (hd->name);
+              if (!rc)
+                rc = cdk_stream_open (hd->idx_name, &hd->idx);
+              if (!rc)
+                _cdk_log_debug ("create key index table\n");
+              if (rc)
+                {
+                  /* this is no real error, it just means we can't create
+                     the index at the given directory. maybe we've no write
+                     access. in this case, we simply disable the index. */
+                  _cdk_log_debug ("disable key index table\n");
+                  rc = 0;
+                  hd->no_cache = 1;
+                }
+            }
+        }
+      else
+        cdk_stream_seek (hd->buf, 0);
     }
-  if (!rc)
-    cdk_stream_seek (kr, kdb->offset);
+  else
+    return CDK_Inv_Mode;
+  
  leave:
   if (rc)
     {
-      cdk_stream_close (kr);
-      kr = NULL;
+      cdk_stream_close (hd->buf);
+      hd->buf = NULL;
     }
   if (ret_kr)
-    *ret_kr = kr;
+    *ret_kr = hd->buf;
   return rc;
 }
 
@@ -321,7 +412,7 @@ find_by_pattern (CDK_KBNODE knode, CDK_DBSEARCH ks)
 {
   CDK_KBNODE node;
   size_t uidlen;
-  char *name;
+  char * name;
   int key_found = 0;
 
   for (node = knode; node; node = node->next)
@@ -335,7 +426,7 @@ find_by_pattern (CDK_KBNODE knode, CDK_DBSEARCH ks)
         case CDK_DBSEARCH_EXACT:
           if (name
               && (strlen (ks->u.pattern) == uidlen
-              || !strncmp (ks->u.pattern, name, uidlen)))
+              && !strncmp (ks->u.pattern, name, uidlen)))
             {
               key_found = 1;
               break;
@@ -359,17 +450,126 @@ find_by_pattern (CDK_KBNODE knode, CDK_DBSEARCH ks)
 }
 
 
+static void
+keydb_cache_free (KEY_TABLE cache)
+{
+  KEY_TABLE c2;
+
+  while (cache)
+    {
+      c2 = cache->next;
+      cache->offset = 0;
+      cdk_keydb_search_free (cache->desc);
+      cdk_free (cache);
+      cache = NULL;
+      cache = c2;
+    }
+}
+
+
+static KEY_TABLE
+keydb_cache_find (KEY_TABLE cache, CDK_DBSEARCH desc)
+{
+  KEY_TABLE t;
+  
+  for (t = cache; t; t = t->next)
+    {
+      if (t->desc->type != desc->type)
+        continue;
+      switch (t->desc->type)
+        {
+        case CDK_DBSEARCH_SHORT_KEYID:
+        case CDK_DBSEARCH_KEYID:
+          if (t->desc->u.keyid[0] == desc->u.keyid[0]
+              && t->desc->u.keyid[1] == desc->u.keyid[1])
+            return t;
+          break;
+
+        case CDK_DBSEARCH_EXACT:
+          if (strlen (t->desc->u.pattern) == strlen (desc->u.pattern)
+              && !strcmp (t->desc->u.pattern, desc->u.pattern))
+            return t;
+          break;
+
+        case CDK_DBSEARCH_SUBSTR:
+          if (strstr (t->desc->u.pattern, desc->u.pattern))
+            return t;
+          break;
+
+        case CDK_DBSEARCH_FPR:
+          if (!memcmp (t->desc->u.fpr, desc->u.fpr, 20))
+            return t;
+          break;
+        }
+    }
+  return NULL;
+}
+  
+
+static int
+keydb_cache_add (CDK_KEYDB_HD hd, CDK_DBSEARCH dbs, u32 offset)
+{
+  KEY_TABLE k;
+
+  if (hd->ncache > KEYDB_CACHE_ENTRIES)
+    return 0;
+  k = cdk_calloc (1, sizeof *k);
+  if (!k)
+    return CDK_Out_Of_Core;
+  k->offset = offset;
+  keydb_search_copy (&k->desc, dbs);
+  k->next = hd->cache;
+  hd->cache = k;
+  hd->ncache++;
+  _cdk_log_debug ("add entry [o=%d t=%d] to the cache\n", offset, dbs->type);
+  return 0;
+}
+
+      
 void
 cdk_keydb_search_reset (CDK_KEYDB_HD hd)
 {
   if (hd)
     {
-      hd->old_offset = 0;
-      hd->offset = 0;
+      keydb_cache_free (hd->cache);
+      hd->cache = NULL;
     }
 }
 
 
+static int
+keydb_search_copy (CDK_DBSEARCH * r_dst, CDK_DBSEARCH src)
+{
+  CDK_DBSEARCH dst;
+  
+  if (!r_dst || !src)
+    return CDK_Inv_Value;
+  dst = cdk_calloc (1, sizeof *dst);
+  if (!dst)
+    return CDK_Out_Of_Core;
+  dst->type = src->type;
+  switch (src->type)
+    {
+    case CDK_DBSEARCH_EXACT:
+    case CDK_DBSEARCH_SUBSTR:
+      dst->u.pattern = cdk_strdup (src->u.pattern);
+      break;
+
+    case CDK_DBSEARCH_SHORT_KEYID:
+    case CDK_DBSEARCH_KEYID:
+      dst->u.keyid[0] = src->u.keyid[0];
+      dst->u.keyid[1] = src->u.keyid[1];
+      break;
+
+    case CDK_DBSEARCH_FPR:
+      memcpy (dst->u.fpr, src->u.fpr, 20);
+      break;
+    }
+  *r_dst = dst;
+  return 0;
+}
+
+  
 int
 cdk_keydb_search_new (CDK_DBSEARCH * r_dbs, int type, void * desc)
 {
@@ -384,7 +584,13 @@ cdk_keydb_search_new (CDK_DBSEARCH * r_dbs, int type, void * desc)
     {
     case CDK_DBSEARCH_EXACT:
     case CDK_DBSEARCH_SUBSTR:
-      c->u.pattern = desc;
+      cdk_free (c->u.pattern);
+      c->u.pattern = cdk_strdup (desc);
+      if (!c->u.pattern)
+        {
+          cdk_free (c);
+          return CDK_Out_Of_Core;
+        }
       break;
 
     case CDK_DBSEARCH_SHORT_KEYID:
@@ -417,49 +623,97 @@ cdk_keydb_search_free (CDK_DBSEARCH dbs)
 {
   if (dbs)
     {
+      if (dbs->type == CDK_DBSEARCH_EXACT || dbs->type == CDK_DBSEARCH_SUBSTR)
+        cdk_free (dbs->u.pattern);
       dbs->type = 0;
       cdk_free (dbs);
     }
 }
 
 
+static int
+keydb_pos_from_cache (CDK_KEYDB_HD hd, CDK_DBSEARCH ks,
+                      int * r_cache_hit, u32 * r_off)
+{
+  KEY_TABLE c;
+  u32 off = 0;
+  int cache_hit = 0;
+
+  if (!hd)
+    return CDK_Inv_Value;
+  
+  c = keydb_cache_find (hd->cache, ks);
+  if (c)
+    {
+      _cdk_log_debug ("found entry in cache.\n");
+      cache_hit = 1;
+      off = c->offset;
+    }
+
+  if (hd->idx && !c)
+    {
+      if (ks->type == CDK_DBSEARCH_KEYID)
+        {
+          if (keydb_idx_search (hd->idx, ks->u.keyid, NULL, &off))
+            return CDK_Error_No_Key;
+          _cdk_log_debug ("found keyid entry in idx table.\n");
+          cache_hit = 1;
+        }
+      else if (ks->type == CDK_DBSEARCH_FPR)
+        {
+          if (keydb_idx_search (hd->idx, NULL, ks->u.fpr, &off))
+            return CDK_Error_No_Key;
+          _cdk_log_debug ("found fpr entry in idx table.\n");
+          cache_hit = 1;
+        }
+    }
+  if (r_off)
+    *r_off = off;
+  if (r_cache_hit)
+    *r_cache_hit = cache_hit;
+  return 0;
+}
+
+
 int
-cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE * r_key)
+cdk_keydb_search (CDK_KEYDB_HD hd, CDK_DBSEARCH ks, CDK_KBNODE * r_key)
 {
   CDK_STREAM kr = NULL;
   CDK_KBNODE knode = NULL;
-  int key_found = 0;
+  int key_found = 0, cache_hit = 0;
   int rc = 0;
+  u32 off = 0;
   size_t pos = 0;
 
-  if (!kdb || !ks)
+  if (!hd || !ks)
     return CDK_Inv_Value;
 
   if (r_key)
     *r_key = NULL;
 
-  rc = cdk_keydb_open (kdb, &kr);
+  rc = cdk_keydb_open (hd, &kr);
+  if (rc)
+    return rc;
+
+  rc = keydb_pos_from_cache (hd, ks, &cache_hit, &off);
   if (rc)
     return rc;
 
   while (!key_found && !rc)
     {
+      if (cache_hit)
+        cdk_stream_seek (kr, off);
       pos = cdk_stream_tell (kr);
-      kdb->old_offset = pos;
       rc = cdk_keydb_get_keyblock (kr, &knode);
       if (rc)
         {
           if (rc == CDK_EOF && knode)
             rc = 0;
+          if (!knode && rc == CDK_EOF)
+            rc = CDK_Error_No_Key;
           if (rc)
             break;
         }
-      if (!knode && rc == CDK_EOF)
-        rc = CDK_Error_No_Key;
-      if (rc)
-	break;
-      pos = cdk_stream_tell (kr);
-      kdb->offset = pos;
 
       switch (ks->type)
         {
@@ -477,14 +731,18 @@ cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE * r_key)
           key_found = find_by_pattern (knode, ks);
 	  break;
 	}
+
       if (key_found)
-        break;
+        {
+          if (!keydb_cache_find (hd->cache, ks))
+            keydb_cache_add (hd, ks, pos);
+          break;
+        }
 
       cdk_kbnode_release (knode);
       knode = NULL;
     }
 
-  cdk_stream_close (kr);
   if (r_key)
     *r_key = key_found? knode : NULL;
   return rc;
@@ -492,48 +750,50 @@ cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE * r_key)
 
 
 int
-cdk_keydb_get_bykeyid (CDK_KEYDB_HD kdb, u32 * keyid, CDK_KBNODE * ret_pk)
+cdk_keydb_get_bykeyid (CDK_KEYDB_HD hd, u32 * keyid, CDK_KBNODE * ret_pk)
 {
   struct cdk_keydb_search_s ks;
 
-  if (!kdb || !keyid || !ret_pk)
+  if (!hd || !keyid || !ret_pk)
     return CDK_Inv_Value;
 
   ks.type = CDK_DBSEARCH_SHORT_KEYID;
   ks.u.keyid[1] = keyid[1];
 
-  return cdk_keydb_search (kdb, &ks, ret_pk);
+  return cdk_keydb_search (hd, &ks, ret_pk);
 }
 
 
 int
-cdk_keydb_get_byfpr (CDK_KEYDB_HD kdb, const byte * fpr, CDK_KBNODE * ret_pk)
+cdk_keydb_get_byfpr (CDK_KEYDB_HD hd, const byte * fpr, CDK_KBNODE * ret_pk)
 {
   struct cdk_keydb_search_s ks;
 
-  if (!kdb || !fpr || !ret_pk)
+  if (!hd || !fpr || !ret_pk)
     return CDK_Inv_Value;
 
   ks.type = CDK_DBSEARCH_FPR;
   memcpy (ks.u.fpr, fpr, 20);
 
-  return cdk_keydb_search (kdb, &ks, ret_pk);
+  return cdk_keydb_search (hd, &ks, ret_pk);
 }
 
 
 int
-cdk_keydb_get_bypattern (CDK_KEYDB_HD kdb, const char *patt,
+cdk_keydb_get_bypattern (CDK_KEYDB_HD hd, const char *patt,
 			 CDK_KBNODE * ret_pk)
 {
   struct cdk_keydb_search_s ks;
+  int rc;
 
-  if (!kdb || !patt || !ret_pk)
+  if (!hd || !patt || !ret_pk)
     return CDK_Inv_Value;
 
   ks.type = CDK_DBSEARCH_SUBSTR;
-  ks.u.pattern = patt;
-
-  return cdk_keydb_search (kdb, &ks, ret_pk);
+  ks.u.pattern = cdk_strdup (patt);
+  rc = cdk_keydb_search (hd, &ks, ret_pk);
+  cdk_free (ks.u.pattern);
+  return rc;
 }
 
 
@@ -598,21 +858,19 @@ _cdk_keydb_find_bykeyid (CDK_KBNODE root, u32 * keyid)
 
 
 int
-cdk_keydb_get_sk_byname (const char * name, cdkPKT_secret_key ** ret_sk,
-			 int alloced)
+_cdk_keydb_get_sk_byname (CDK_KEYDB_HD hd, const char * name,
+                          cdkPKT_secret_key ** ret_sk, int alloced)
 {
   CDK_KBNODE knode = NULL, node = NULL;
   cdkPKT_secret_key * sk = NULL;
-  CDK_KEYDB_HD hd;
   int req_usage = 0;
   int rc = 0;
 
   if (!ret_sk)
     return CDK_Inv_Value;
-
-  hd = cdk_keydb_get_ctx (1, -1);
   if (!hd)
     return CDK_Error_No_Keyring;
+  
   rc = cdk_keydb_search_auto (hd, name, &knode);
   if (rc)
     goto leave;
@@ -636,29 +894,26 @@ cdk_keydb_get_sk_byname (const char * name, cdkPKT_secret_key ** ret_sk,
   cdk_kbnode_release (knode);
 
 leave:
-  cdk_keydb_search_reset (hd);
   *ret_sk = sk;
   return rc;
 }
 
 
 int
-cdk_keydb_get_pk_byname (const char * name, cdkPKT_public_key ** ret_pk,
-			 int alloced)
+_cdk_keydb_get_pk_byname (CDK_KEYDB_HD hd, const char * name,
+                          cdkPKT_public_key ** ret_pk, int alloced)
 {
   CDK_KBNODE knode, node = NULL;
-  cdkPKT_public_key *pk = NULL;
-  CDK_KEYDB_HD hd;
-  const char *s;
+  cdkPKT_public_key * pk = NULL;
+  const char * s;
   int req_usage = 0;
   int rc = 0;
 
   if (!name)
     return CDK_Inv_Value;
-
-  hd = cdk_keydb_get_ctx (0, -1);
   if (!hd)
     return CDK_Error_No_Keyring;
+  
   rc = cdk_keydb_search_auto (hd, name, &knode);
   if (rc)
     goto leave;
@@ -693,7 +948,6 @@ cdk_keydb_get_pk_byname (const char * name, cdkPKT_public_key ** ret_pk,
   cdk_kbnode_release (knode);
 
 leave:
-  cdk_keydb_search_reset (hd);
   if (ret_pk)
     *ret_pk = pk;
   return rc;
@@ -704,22 +958,14 @@ int
 cdk_keydb_get_pk (CDK_KEYDB_HD hd, u32 * keyid, cdkPKT_public_key ** r_pk)
 {
   struct cdk_keydb_search_s ks;
-  CDK_KBNODE key = NULL, node = NULL;
-  CDK_KEYDB_HD keydb_hd = NULL;
-  cdkPKT_public_key *pk = NULL;
+  CDK_KBNODE knode = NULL, node = NULL;
+  cdkPKT_public_key * pk = NULL;
   int rc = 0;
 
-  if (!keyid || !r_pk)
+  if (!keyid)
     return CDK_Inv_Value;
-
-  if (hd == NULL)
-    {
-      keydb_hd = cdk_keydb_get_ctx (0, -1);
-      if (!keydb_hd)
-	return CDK_Error_No_Keyring;
-    }
-  else
-    keydb_hd = hd;
+  if (!hd)
+    return CDK_Error_No_Keyring;
 
   if (!keyid[0])
     {
@@ -732,20 +978,21 @@ cdk_keydb_get_pk (CDK_KEYDB_HD hd, u32 * keyid, cdkPKT_public_key ** r_pk)
       ks.u.keyid[0] = keyid[0];
       ks.u.keyid[1] = keyid[1];
     }
-  rc = cdk_keydb_search (keydb_hd, &ks, &key);
+  rc = cdk_keydb_search (hd, &ks, &knode);
   if (rc)
     goto leave;
-  node = _cdk_keydb_find_bykeyid (key, keyid);
+  node = _cdk_keydb_find_bykeyid (knode, keyid);
   if (!node)
     {
       rc = CDK_Error_No_Key;
       goto leave;
     }
   _cdk_copy_pubkey (&pk, node->pkt->pkt.public_key);
-  cdk_kbnode_release (key);
+  cdk_kbnode_release (knode);
 
 leave:
-  *r_pk = pk;
+  if (r_pk)
+    *r_pk = pk;
   return rc;
 }
 
@@ -753,19 +1000,15 @@ leave:
 int
 cdk_keydb_get_sk (CDK_KEYDB_HD hd, u32 * keyid, cdkPKT_secret_key ** ret_sk)
 {
-  CDK_KEYDB_HD khd;
   CDK_KBNODE snode, node;
-  cdkPKT_secret_key *sk = NULL;
+  cdkPKT_secret_key * sk = NULL;
   int rc = 0;
 
+  if (!keyid)
+    return CDK_Inv_Value;
   if (!hd)
-    {
-      khd = cdk_keydb_get_ctx (1, -1);
-      if (!khd)
-	return CDK_Error_No_Keyring;
-    }
-  else
-    khd = hd;
+    return CDK_Error_No_Keyring;
+
   rc = cdk_keydb_get_bykeyid (hd, keyid, &snode);
   if (rc)
     goto leave;
@@ -782,7 +1025,8 @@ cdk_keydb_get_sk (CDK_KEYDB_HD hd, u32 * keyid, cdkPKT_secret_key ** ret_sk)
   cdk_kbnode_release (snode);
 
 leave:
-  *ret_sk = sk;
+  if (ret_sk)
+    *ret_sk = sk;
   return rc;
 }
 
@@ -791,10 +1035,10 @@ static int
 keydb_merge_selfsig (CDK_KBNODE key)
 {
   CDK_KBNODE k, kbnode, unode;
-  struct cdk_subpkt_s *s = NULL;
-  cdkPKT_signature *sig = NULL;
-  cdkPKT_user_id *uid = NULL;
-  const byte *symalg = NULL, *hashalg = NULL, *compalg = NULL;
+  struct cdk_subpkt_s * s = NULL;
+  cdkPKT_signature * sig = NULL;
+  cdkPKT_user_id * uid = NULL;
+  const byte * symalg = NULL, * hashalg = NULL, * compalg = NULL;
   size_t nsymalg = 0, nhashalg = 0, ncompalg = 0, n = 0;
   int key_usage = 0, key_expire = 0;
 
@@ -908,7 +1152,7 @@ keydb_parse_allsigs (CDK_KBNODE knode, int check)
   cdkPKT_signature * sig;
   cdkPKT_public_key * pk;
   struct cdk_subpkt_s * s = NULL;
-  u32 expiredate = 0, curtime = _cdk_timestamp ();
+  u32 expiredate = 0, curtime = _cdk_timestamp (), keyid[2];
   int rc = 0;
 
   if (!knode)
@@ -928,6 +1172,11 @@ keydb_parse_allsigs (CDK_KBNODE knode, int check)
 	k->pkt->pkt.public_key->is_revoked = 0;
     }
 
+  kb = cdk_kbnode_find (knode, CDK_PKT_PUBLIC_KEY);
+  if (!kb)
+    return CDK_Inv_Packet;
+  cdk_pk_get_keyid (kb->pkt->pkt.public_key, keyid);
+  
   for (k = knode; k; k = k->next)
     {
       if (k->pkt->pkttype == CDK_PKT_SIGNATURE)
@@ -962,6 +1211,8 @@ keydb_parse_allsigs (CDK_KBNODE knode, int check)
 	  /* Revocation certifcates for user ID's */
 	  else if (sig->sig_class == 0x30)
 	    {
+              if (sig->keyid[0] != keyid[0] || sig->keyid[1] != keyid[1])
+                continue; /* revokes an earlier signature, no userID. */
 	      kb = cdk_kbnode_find_prev (knode, k, CDK_PKT_USER_ID);
 	      if (kb)
 		{
@@ -1032,7 +1283,7 @@ keydb_parse_allsigs (CDK_KBNODE knode, int check)
 int
 cdk_keydb_get_keyblock (CDK_STREAM inp, CDK_KBNODE * r_knode)
 {
-  CDK_PACKET *pkt = NULL;
+  CDK_PACKET * pkt = NULL;
   CDK_KBNODE knode = NULL, node = NULL;
   u32 keyid[2], main_keyid[2];
   int rc = 0, rc2 = 0, old_off;
@@ -1126,7 +1377,7 @@ classify_data (const byte * buf, size_t len)
       buf += 2;
       len -= 2;
     }
-  
+
   if (len == 8 || len == 16 || len == 40)
     {
       for (i = 0; i < len; i++)
@@ -1157,7 +1408,10 @@ keyid_from_str (const char * buf, u32 * keyid)
     buf += 2;
 
   if (strlen (buf) == 8)
-    keyid[1] = strtoul (buf, NULL, 16);
+    {
+      keyid[0] = 0;
+      keyid[1] = strtoul (buf, NULL, 16);
+    }
   else if (strlen (buf) == 16)
     {
       keyid[0] = strtoul (buf, NULL, 16);
@@ -1192,9 +1446,10 @@ cdk_keydb_search_auto (CDK_KEYDB_HD hd, const char *desc, CDK_KBNODE * r_knode)
 {
   u32 keyid[2];
   byte fpr[20];
-  int rc;
-
-  switch (classify_data (desc, strlen (desc)))
+  int rc, type;
+  
+  type = classify_data (desc, strlen (desc));
+  switch (type)
     {
     case CDK_DBSEARCH_EXACT:
     case CDK_DBSEARCH_SUBSTR:
@@ -1214,24 +1469,20 @@ cdk_keydb_search_auto (CDK_KEYDB_HD hd, const char *desc, CDK_KBNODE * r_knode)
 
     default:
       rc = CDK_Inv_Mode;
+      break;
     }
   return rc;
 }
 
 
 int
-cdk_keydb_export (CDK_STREAM out, CDK_STRLIST remusr, int secret)
+cdk_keydb_export (CDK_KEYDB_HD hd, CDK_STREAM out, CDK_STRLIST remusr)
 {
-  CDK_KEYDB_HD hd;
   CDK_KBNODE knode, node;
   CDK_STRLIST r;
   int old_ctb = 0;
   int rc = 0;
 
-  hd = cdk_keydb_get_ctx (secret, -1);
-  if (!hd)
-    return CDK_Error_No_Keyring;
-
   for (r = remusr; r; r = r->next)
     {
       rc = cdk_keydb_search_auto (hd, r->d, &knode);
@@ -1242,13 +1493,13 @@ cdk_keydb_export (CDK_STREAM out, CDK_STRLIST remusr, int secret)
 	  /* those packets are not intended for the real wolrd */
 	  if (node->pkt->pkttype == CDK_PKT_RING_TRUST)
 	    continue;
-          if (node->pkt->pkttype == CDK_PKT_PUBLIC_KEY
-              && node->pkt->pkt.public_key->version == 3)
-            old_ctb = 1;
 	  /* we never export local signed signatures */
 	  if (node->pkt->pkttype == CDK_PKT_SIGNATURE &&
 	      !node->pkt->pkt.signature->flags.exportable)
             continue;
+          if (node->pkt->pkttype == CDK_PKT_PUBLIC_KEY
+              && node->pkt->pkt.public_key->version == 3)
+            old_ctb = 1;
           node->pkt->old_ctb = old_ctb;
 	  rc = cdk_pkt_build (out, node->pkt);
 	  if (rc)
@@ -1286,20 +1537,26 @@ cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode, int result[4])
   u32 keyid[2];
   int rc = 0, is_sk = 0;
 
+  if (!hd || !knode)
+    return CDK_Inv_Value;
+  
   memset (result, 0, 4 * sizeof (int));
   pkt = find_key_packet (knode, &is_sk);
   if (!pkt)
       return CDK_Inv_Packet;
   result[is_sk] = 1;
-  cdk_pk_get_keyid (pkt->pkt.public_key, keyid);
+  if (is_sk)
+    cdk_sk_get_keyid (pkt->pkt.secret_key, keyid);
+  else
+    cdk_pk_get_keyid (pkt->pkt.public_key, keyid);
   cdk_keydb_get_bykeyid (hd, keyid, &chk);
   if (chk)
     {
+      /* fixme: search for new signatures */
       cdk_kbnode_release (chk);
       return 0;
     }
   
-  cdk_keydb_search_reset (hd);
   if (hd->buf)
     return CDK_Inv_Mode;
   rc = _cdk_stream_append (hd->name, &out);
@@ -1317,44 +1574,40 @@ cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode, int result[4])
   if (!rc)
     result[is_sk? 3 : 2] = 1;
   cdk_stream_close (out);
+  if (!hd->no_cache)
+    keydb_idx_rebuild (hd);  
   return rc;
 }  
 
 
 /* internal functions */
 CDK_KBNODE
-_cdk_keydb_get_pkblock (u32 * keyid)
+_cdk_keydb_get_pkblock (CDK_KEYDB_HD hd, u32 * keyid)
 {
-  CDK_KEYDB_HD hd;
   CDK_KBNODE kbnode = NULL;
-  int rc = 0;
+  int rc;
 
-  hd = cdk_keydb_get_ctx (0, -1);
   if (hd)
     {
       rc = cdk_keydb_get_bykeyid (hd, keyid, &kbnode);
       if (rc)
         kbnode = NULL;
-      cdk_keydb_search_reset (hd);
     }
   return kbnode;
 }
 
 
 CDK_KBNODE
-_cdk_keydb_get_skblock (u32 * keyid)
+_cdk_keydb_get_skblock (CDK_KEYDB_HD hd, u32 * keyid)
 {
   CDK_KBNODE snode = NULL;
-  CDK_KEYDB_HD hd;
-  int rc = 0;
+  int rc;
 
-  hd = cdk_keydb_get_ctx (1, -1);
   if (hd)
     {
       rc = cdk_keydb_get_bykeyid (hd, keyid, &snode);
       if (rc)
         snode = NULL;
-      cdk_keydb_search_reset (hd);
     }
   return snode;
 }
