@@ -19,7 +19,7 @@
  */
 
 #ifdef HAVE_CONFIG_H
- #include <config.h>
+# include <config.h>
 #endif
 #include <stdio.h>
 
@@ -28,15 +28,20 @@
 #include "main.h"
 #include "types.h"
 
+struct one_key_s
+{
+  u32 expire_date;
+  int algo;
+  int len;
+  GCRY_MPI resarr[6];
+  size_t n;
+  cdkPKT_public_key * pk;
+  cdkPKT_secret_key * sk;
+};
+  
 struct cdk_keygen_s
 {
-  long expire_date;
-  int prim_algo;
-  int prim_len;
-  GCRY_MPI prim_resarr[6];
-  size_t prim_n;
-  cdkPKT_public_key * prim_pk;
-  cdkPKT_secret_key * prim_sk;
+  struct one_key_s prim;
   char * user_id;
   cdkPKT_user_id * id;
   byte * sym_prefs;
@@ -46,15 +51,11 @@ struct cdk_keygen_s
   byte * zip_prefs;
   size_t zip_len;
   unsigned mdc_feature:1;
-  int sub_algo;
-  int sub_len;
-  GCRY_MPI sub_resarr[6];
-  size_t sub_n;
-  cdkPKT_public_key * sub_pk;
-  cdkPKT_secret_key * sub_sk;
+  struct one_key_s sub;
 };
 
 
+/* default preferences */
 static byte def_sym_prefs[] = {GCRY_CIPHER_AES128, GCRY_CIPHER_CAST5,
                                GCRY_CIPHER_TWOFISH, GCRY_CIPHER_AES192,
                                GCRY_CIPHER_AES256, GCRY_CIPHER_3DES,
@@ -148,26 +149,26 @@ cdk_keygen_set_algo_info (CDK_KEYGEN_CTX hd, int type, int algo, int bits)
 
   if (!type) /* primary */
     {
-      hd->prim_algo = algo;
-      hd->prim_len = bits;
+      hd->prim.algo = algo;
+      hd->prim.len = bits;
     }
   else
     {
-      hd->sub_algo = algo;
-      hd->sub_len = bits;
+      hd->sub.algo = algo;
+      hd->sub.len = bits;
     }
 
   /* sanity checks for minimal and maxmimal key length. */
-  if (hd->prim_algo == GCRY_PK_DSA && hd->prim_len > 1024)
-    hd->prim_len = 1024;
-  if (hd->prim_len < 768)
-    hd->prim_len = 768;
-  if (hd->sub_len < 768)
-    hd->sub_len = 768;
-  if (hd->prim_len > 4096)
-    hd->prim_len = 4096;
-  if (hd->sub_len > 4096)
-    hd->prim_len = 4096;
+  if (hd->prim.algo == GCRY_PK_DSA && hd->prim.len > 1024)
+    hd->prim.len = 1024;
+  if (hd->prim.len < 768)
+    hd->prim.len = 768;
+  if (hd->sub.len < 768)
+    hd->sub.len = 768;
+  if (hd->prim.len > 4096)
+    hd->prim.len = 4096;
+  if (hd->sub.len > 4096)
+    hd->prim.len = 4096;
 
   return 0;
 }
@@ -192,14 +193,20 @@ cdk_keygen_set_mdc_feature (CDK_KEYGEN_CTX hd, int val)
 /**
  * cdk_keygen_set_expire_date: set the expire date of the primary key
  * @hd: keygen object
+ * @type: key type (0=primary, 1=seconardy)
  * @timestamp: the date the key should expire
  *
  **/
 void
-cdk_keygen_set_expire_date (CDK_KEYGEN_CTX hd, long timestamp)
+cdk_keygen_set_expire_date (CDK_KEYGEN_CTX hd, int type, long timestamp)
 {
   if (hd)
-    hd->expire_date = timestamp;
+    {
+      if (!type)
+        hd->prim.expire_date = timestamp;
+      else
+        hd->sub.expire_date = timestamp;
+    }
 }
 
 
@@ -269,15 +276,15 @@ read_rsa_key (GCRY_SEXP s_key, GCRY_MPI * resarr)
 static int
 generate_subkey (CDK_KEYGEN_CTX hd)
 {
-  GCRY_SEXP s_params, s_key;
-  size_t n = hd->sub_len;
+  GCRY_SEXP s_params = NULL, s_key;
+  size_t n = hd->sub.len;
   int rc;
 
-  if (is_DSA (hd->sub_algo))
+  if (is_DSA (hd->sub.algo))
     rc = gcry_sexp_build (&s_params, NULL, "(genkey(dsa(nbits %d)))", n);
-  else if (is_ELG (hd->sub_algo))
+  else if (is_ELG (hd->sub.algo))
     rc = gcry_sexp_build (&s_params, NULL, "(genkey(elg(nbits %d)))", n);
-  else if (is_RSA (hd->sub_algo))
+  else if (is_RSA (hd->sub.algo))
     rc = gcry_sexp_build (&s_params, NULL, "(genkey(rsa(nbits %d)))", n);
   else
       rc = CDK_Inv_Algo;
@@ -286,12 +293,12 @@ generate_subkey (CDK_KEYGEN_CTX hd)
   gcry_sexp_release (s_params);
   if (!rc)
     {
-      if (is_DSA (hd->sub_algo))
-        rc = read_dsa_key (s_key, hd->sub_resarr);
-      else if (is_ELG (hd->sub_algo))
-        rc = read_elg_key (s_key, hd->sub_resarr);
-      else if (is_RSA (hd->sub_algo))
-        rc = read_rsa_key (s_key, hd->sub_resarr);
+      if (is_DSA (hd->sub.algo))
+        rc = read_dsa_key (s_key, hd->sub.resarr);
+      else if (is_ELG (hd->sub.algo))
+        rc = read_elg_key (s_key, hd->sub.resarr);
+      else if (is_RSA (hd->sub.algo))
+        rc = read_rsa_key (s_key, hd->sub.resarr);
     }
   gcry_sexp_release (s_key);
   return rc;
@@ -307,15 +314,15 @@ int
 cdk_keygen_start (CDK_KEYGEN_CTX hd)
 {
   GCRY_SEXP s_params = NULL, s_key = NULL;
-  size_t n = hd->prim_len;
+  size_t n = hd->prim.len;
   int rc = 0;
   
-  if (!hd)
+  if (!hd || !hd->user_id)
     return CDK_Inv_Value;
-  if (is_ELG (hd->prim_algo))
+  if (is_ELG (hd->prim.algo))
     return CDK_Inv_Mode;
-  if (!hd->prim_len)
-    hd->prim_len = 1024;
+  if (!hd->prim.len)
+    hd->prim.len = 1024;
 
   if (!hd->sym_prefs)
     cdk_keygen_set_prefs (hd, CDK_PREFTYPE_SYM, NULL, 0);
@@ -324,9 +331,9 @@ cdk_keygen_start (CDK_KEYGEN_CTX hd)
   if (!hd->zip_prefs)
     cdk_keygen_set_prefs (hd, CDK_PREFTYPE_ZIP, NULL, 0);
 
-  if (is_DSA (hd->prim_algo))
+  if (is_DSA (hd->prim.algo))
     rc = gcry_sexp_build (&s_params, NULL, "(genkey(dsa(nbits %d)))", n);
-  else if (is_RSA (hd->prim_algo))
+  else if (is_RSA (hd->prim.algo))
     rc = gcry_sexp_build (&s_params, NULL, "(genkey(rsa(nbits %d)))", n);
   else
     rc = CDK_Inv_Algo;
@@ -335,19 +342,19 @@ cdk_keygen_start (CDK_KEYGEN_CTX hd)
   gcry_sexp_release (s_params);
   if (!rc)
     {
-      if (is_DSA (hd->prim_algo))
-        rc = read_dsa_key (s_key, hd->prim_resarr);
-      else if (is_RSA (hd->prim_algo))
-        rc = read_rsa_key (s_key, hd->prim_resarr);
-      hd->prim_n = cdk_pk_get_npkey (hd->prim_algo);
+      if (is_DSA (hd->prim.algo))
+        rc = read_dsa_key (s_key, hd->prim.resarr);
+      else if (is_RSA (hd->prim.algo))
+        rc = read_rsa_key (s_key, hd->prim.resarr);
+      hd->prim.n = cdk_pk_get_npkey (hd->prim.algo);
     }
   gcry_sexp_release (s_key);
 
   if (!rc)
     {
-      if (hd->sub_algo && hd->sub_len)
+      if (hd->sub.algo && hd->sub.len)
         rc = generate_subkey (hd);
-      hd->sub_n = cdk_pk_get_npkey (hd->sub_algo);
+      hd->sub.n = cdk_pk_get_npkey (hd->sub.algo);
     }
   return rc;
 }
@@ -366,7 +373,7 @@ gcry_mpi_to_native (CDK_KEYGEN_CTX hd, size_t nkey, int type,
   if (!pk && !sk)
     return CDK_Inv_Value;
 
-  resarr = type? hd->sub_resarr : hd->prim_resarr;
+  resarr = type? hd->sub.resarr : hd->prim.resarr;
   if (sk)
     i += cdk_pk_get_npkey (sk->pubkey_algo);
   while (j != nkey)
@@ -403,11 +410,11 @@ pk_create (CDK_KEYGEN_CTX hd, int type)
   if (!pk)
     return NULL;
   pk->version = 4;
-  pk->pubkey_algo = type? hd->sub_algo : hd->prim_algo;
+  pk->pubkey_algo = type? hd->sub.algo : hd->prim.algo;
   pk->timestamp = _cdk_timestamp ();
-  if (hd->expire_date)
-    pk->expiredate = (hd->expire_date - pk->timestamp) / 86400;
-  rc = gcry_mpi_to_native (hd, type? hd->sub_n: hd->prim_n, type, pk, NULL);
+  if (hd->prim.expire_date)
+    pk->expiredate = (hd->prim.expire_date - pk->timestamp) / 86400;
+  rc = gcry_mpi_to_native (hd, type? hd->sub.n: hd->prim.n, type, pk, NULL);
   if (rc)
     {
       cdk_free (pk);
@@ -428,7 +435,7 @@ sk_create (CDK_KEYGEN_CTX hd, int type, cdkPKT_public_key * pk)
     return NULL;
   _cdk_copy_pubkey (&sk->pk, pk);
   sk->version = 4;
-  sk->pubkey_algo = type? hd->sub_algo : hd->prim_algo;
+  sk->pubkey_algo = type? hd->sub.algo : hd->prim.algo;
   sk->csum = 0;
   sk->is_protected = 0;
   nskey = cdk_pk_get_nskey (sk->pubkey_algo);
@@ -511,7 +518,7 @@ sig_self_create (CDK_KEYGEN_CTX hd,
   sig->version = 4;
   sig->timestamp = _cdk_timestamp ();
   sig->sig_class = 0x13;
-  sig->pubkey_algo = hd->prim_algo;
+  sig->pubkey_algo = hd->prim.algo;
   sig->digest_algo = GCRY_MD_SHA1;
 
   sig->hashed = node = cdk_subpkt_new (4);
@@ -568,7 +575,7 @@ sig_self_create (CDK_KEYGEN_CTX hd,
         {
           node->size = 4;
           node->type = SIGSUBPKT_KEY_EXPIRE;
-          _cdk_u32tobuf ((hd->expire_date - pk->timestamp), node->d);
+          _cdk_u32tobuf ((hd->prim.expire_date - pk->timestamp), node->d);
           _cdk_subpkt_add (sig->hashed, node);
         }
     }
@@ -619,11 +626,11 @@ cdk_keygen_save (CDK_KEYGEN_CTX hd, const char * pub, const char * sec)
   CDK_PACKET pkt;
   int rc;
 
-  hd->prim_pk = pk_create (hd, 0);
-  if (!hd->prim_pk)
+  hd->prim.pk = pk_create (hd, 0);
+  if (!hd->prim.pk)
     return CDK_Inv_Packet;
-  hd->prim_sk = sk_create (hd, 0, hd->prim_pk);
-  if (!hd->prim_sk)
+  hd->prim.sk = sk_create (hd, 0, hd->prim.pk);
+  if (!hd->prim.sk)
     return CDK_Inv_Packet;
   hd->id = uid_create (hd);
   if (!hd->id)
@@ -635,7 +642,7 @@ cdk_keygen_save (CDK_KEYGEN_CTX hd, const char * pub, const char * sec)
   
   init_packet (&pkt);
   pkt.pkttype = CDK_PKT_PUBLIC_KEY;
-  pkt.pkt.public_key = hd->prim_pk;
+  pkt.pkt.public_key = hd->prim.pk;
   rc = cdk_pkt_build (out, &pkt);
   if (rc)
     goto fail;
@@ -649,25 +656,25 @@ cdk_keygen_save (CDK_KEYGEN_CTX hd, const char * pub, const char * sec)
 
   init_packet (&pkt);
   pkt.pkttype = CDK_PKT_SIGNATURE;
-  pkt.pkt.signature = sig_self_create (hd, hd->prim_pk, hd->prim_sk, hd->id);
+  pkt.pkt.signature = sig_self_create (hd, hd->prim.pk, hd->prim.sk, hd->id);
   rc = cdk_pkt_build (out, &pkt);
   cdk_pkt_free (&pkt);
   if (rc)
     goto fail;
 
-  if (hd->sub_algo)
+  if (hd->sub.algo)
     {
       init_packet (&pkt);
       pkt.pkttype = CDK_PKT_PUBLIC_SUBKEY;
-      pkt.pkt.public_key = hd->sub_pk = pk_create (hd, 1);
+      pkt.pkt.public_key = hd->sub.pk = pk_create (hd, 1);
       rc = cdk_pkt_build (out, &pkt);
       if (rc)
         goto fail;
 
       init_packet (&pkt);
       pkt.pkttype = CDK_PKT_SIGNATURE;
-      pkt.pkt.signature = sig_subkey_create (hd, hd->prim_pk, hd->sub_pk,
-                                             hd->prim_sk);
+      pkt.pkt.signature = sig_subkey_create (hd, hd->prim.pk, hd->sub.pk,
+                                             hd->prim.sk);
       rc = cdk_pkt_build (out, &pkt);
       cdk_pkt_free (&pkt);
       if (rc)
@@ -682,7 +689,7 @@ cdk_keygen_save (CDK_KEYGEN_CTX hd, const char * pub, const char * sec)
 
   init_packet (&pkt);
   pkt.pkttype = CDK_PKT_SECRET_KEY;
-  pkt.pkt.secret_key = hd->prim_sk;
+  pkt.pkt.secret_key = hd->prim.sk;
   rc = cdk_pkt_build (out, &pkt);
   if (rc)
     goto fail;
@@ -692,11 +699,11 @@ cdk_keygen_save (CDK_KEYGEN_CTX hd, const char * pub, const char * sec)
   pkt.pkt.user_id = hd->id;
   rc = cdk_pkt_build (out, &pkt);
 
-  if (hd->sub_algo)
+  if (hd->sub.algo)
     {
       init_packet (&pkt);
       pkt.pkttype = CDK_PKT_SECRET_SUBKEY;
-      pkt.pkt.secret_key = hd->sub_sk = sk_create (hd, 1, hd->sub_pk);
+      pkt.pkt.secret_key = hd->sub.sk = sk_create (hd, 1, hd->sub.pk);
       rc = cdk_pkt_build (out, &pkt);
       if (rc)
         goto fail;
@@ -719,15 +726,16 @@ cdk_keygen_free (CDK_KEYGEN_CTX hd)
 {
   if (hd)
     {
-      _cdk_free_pubkey (hd->prim_pk);
-      _cdk_free_pubkey (hd->sub_pk);
-      _cdk_free_seckey (hd->prim_sk);
-      _cdk_free_seckey (hd->sub_sk);
+      _cdk_free_pubkey (hd->prim.pk);
+      _cdk_free_pubkey (hd->sub.pk);
+      _cdk_free_seckey (hd->prim.sk);
+      _cdk_free_seckey (hd->sub.sk);
       _cdk_free_userid (hd->id);
       cdk_free (hd->sym_prefs);
       cdk_free (hd->hash_prefs);
       cdk_free (hd->zip_prefs);
-      _cdk_free_mpi_array (hd->prim_n, hd->prim_resarr);
+      _cdk_free_mpi_array (hd->prim.n, hd->prim.resarr);
+      _cdk_free_mpi_array (hd->sub.n, hd->sub.resarr);
       cdk_free (hd);
     }
 }
