@@ -41,15 +41,6 @@ pklist_key_is_present (CDK_KEYLIST pkl, cdkPKT_public_key * pk)
 }
 
 
-static int
-pklist_get_pk (const char * remusr, cdkPKT_public_key ** ret_pk)
-{
-  if (!remusr || !ret_pk)
-    return CDK_Inv_Value;
-  return cdk_keydb_get_pk_byname (remusr, ret_pk, 1);
-}
-
-
 /* Here we check if *all* keys have the MDC feature. Even if one
    key doesn't support it, it is not used. */
 static int
@@ -223,12 +214,16 @@ cdk_pklist_release (CDK_KEYLIST pkl)
 
 
 int
-cdk_pklist_build (CDK_STRLIST remusr, CDK_KEYLIST * ret_pkl, int use)
+cdk_pklist_build (CDK_KEYDB_HD hd, CDK_STRLIST remusr,
+                  CDK_KEYLIST * ret_pkl, int use)
 {
   CDK_KEYLIST pkl = NULL, r = NULL;
-  cdkPKT_public_key *pk = NULL;
+  cdkPKT_public_key * pk = NULL;
   int rc = 0;
 
+  if (!hd)
+    return CDK_Inv_Value;
+  
   for (; remusr; remusr = remusr->next)
     {
       pk = cdk_calloc (1, sizeof *pk);
@@ -238,7 +233,7 @@ cdk_pklist_build (CDK_STRLIST remusr, CDK_KEYLIST * ret_pkl, int use)
 	  goto leave;
 	}
       pk->pubkey_usage = use;
-      rc = pklist_get_pk (remusr->d, &pk);
+      rc = _cdk_keydb_get_pk_byname (hd, remusr->d, &pk, 1);
       if (rc)
 	goto leave;
       else
@@ -339,14 +334,14 @@ cdk_sklist_release (CDK_KEYLIST skl)
 
 
 int
-cdk_sklist_build (CDK_STRLIST locusr, CDK_KEYLIST * ret_skl, int unlock,
-		  unsigned int use)
+cdk_sklist_build (CDK_KEYDB_HD hd, CDK_STRLIST locusr,
+                  CDK_KEYLIST * ret_skl, int unlock, unsigned int use)
 {
   CDK_KEYLIST r = NULL, skl = NULL;
   cdkPKT_secret_key * sk = NULL;
   int rc = 0;
 
-  if (!ret_skl)
+  if (!hd || !ret_skl)
     return CDK_Inv_Value;
 
   if (!locusr)  /* use the default one */
@@ -355,7 +350,7 @@ cdk_sklist_build (CDK_STRLIST locusr, CDK_KEYLIST * ret_skl, int unlock,
       if (!sk)
 	return CDK_Out_Of_Core;
       sk->req_usage = use;
-      rc = cdk_keydb_get_sk_byname (NULL, &sk, 1);
+      rc = _cdk_keydb_get_sk_byname (hd, NULL, &sk, 1);
       if (rc)
 	{
 	  _cdk_free_seckey (sk);
@@ -390,7 +385,7 @@ cdk_sklist_build (CDK_STRLIST locusr, CDK_KEYLIST * ret_skl, int unlock,
 	  if (!sk)
 	    return CDK_Out_Of_Core;
 	  sk->req_usage = use;
-	  rc = cdk_keydb_get_sk_byname (locusr->d, &sk, 1);
+	  rc = _cdk_keydb_get_sk_byname (hd, locusr->d, &sk, 1);
 	  if (rc)
 	    {
 	      _cdk_free_seckey (sk);
@@ -439,6 +434,9 @@ cdk_sklist_write_onepass (CDK_KEYLIST skl, CDK_STREAM outp, int sigclass)
   if (!skl || !outp)
     return CDK_Inv_Value;
 
+  if (skl->type != CDK_PKT_SECRET_KEY)
+    return CDK_Inv_Mode;
+  
   for (skcount = 0, r = skl; r; r = r->next)
     skcount++;
 
@@ -487,6 +485,9 @@ cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, GCRY_MD_HD hash,
   if (!skl || !outp || !hash)
     return CDK_Inv_Value;
 
+  if (skl->type != CDK_PKT_SECRET_KEY)
+    return CDK_Inv_Mode;
+  
   for (r = skl; r; r = r->next)
     {
       sig = cdk_calloc (1, sizeof *sig);
