@@ -66,7 +66,7 @@ init_sockets (void)
   
 static int
 keyserver_handle_hkp (const char * host, int port,
-                      u32 * keyid, CDK_KBNODE * r_knode)
+                      u32 keyid, CDK_KBNODE * r_knode)
 {
   struct hostent * hp;
   struct sockaddr_in saddr;
@@ -107,7 +107,7 @@ keyserver_handle_hkp (const char * host, int port,
             "GET /pks/lookup?op=get&search=0x%08lX HTTP/1.0\r\n"
             "Host: %s:%d\r\n"
             "\r\n",
-            keyid[0], host, port);
+            keyid, host, port);
   _cdk_log_debug ("%s\n", buf);
 
   nwritten = send (fd, buf, strlen (buf), 0);
@@ -158,7 +158,7 @@ keyserver_handle_hkp (const char * host, int port,
 
 static int
 keyserver_handle_ldap (const char * host, int port,
-                       u32 * keyid, CDK_KBNODE * r_knode)
+                       u32 keyid, CDK_KBNODE * r_knode)
 {
   return CDK_Not_Implemented;
 }
@@ -169,26 +169,33 @@ keyserver_handle_ldap (const char * host, int port,
  * @host: URL or hostname of the keyserver
  * @port: The used port
  * @keyid: KeyID of the key to retrieve
+ * @kid_type: KeyID type (long, short, fingerprint)
  * @r_knode: The key that was found wrapped in a KBNODE struct
  **/
-int
-cdk_keyserver_recv_key (const char * host, int port,
-                        u32 * keyid, CDK_KBNODE * r_knode)
+int cdk_keyserver_recv_key (const char * host, int port,
+                            const byte * keyid, int kid_type,
+                            CDK_KBNODE * r_knode)
 {
   int proto;
   int rc = 0;
+  u32 kid = 0;
 
   if (!initialized)
     init_sockets ();
   
   if (!port)
     port = 11371;
-
-  if (!strncmp (host, "http://", 7) || !strncmp (host, "hkp://", 6))
+  
+  if (!strncmp (host, "http://", 7))
     {
       proto = KEYSERVER_HKP;
       host += 7;
     }
+  else if (!strncmp (host, "hkp://", 6))
+    {
+      proto = KEYSERVER_HKP;
+      host += 6;
+    }
   else if (!strncmp (host, "ldap://", 7))
     {
       proto = KEYSERVER_LDAP;
@@ -196,15 +203,33 @@ cdk_keyserver_recv_key (const char * host, int port,
     }
   else
     proto = KEYSERVER_HKP;
+
+  switch (kid_type)
+    {
+    case CDK_DBSEARCH_SHORT_KEYID:
+      kid = _cdk_buftou32 (keyid);
+      break;
+      
+    case CDK_DBSEARCH_KEYID:
+      kid = _cdk_buftou32 (keyid + 4);
+      break;
+      
+    case CDK_DBSEARCH_FPR:
+      kid = _cdk_buftou32 (keyid + 16);
+      break;
+      
+    default:
+      return CDK_Inv_Mode;
+    }
   
   switch (proto)
     {
     case KEYSERVER_HKP:
-      rc = keyserver_handle_hkp (host, port, keyid, r_knode);
+      rc = keyserver_handle_hkp (host, port, kid, r_knode);
       break;
 
     case KEYSERVER_LDAP:
-      rc = keyserver_handle_ldap (host, port, keyid, r_knode);
+      rc = keyserver_handle_ldap (host, port, kid, r_knode);
       break;
     }
 
