@@ -74,7 +74,7 @@ cdk_strerror (int ec)
     case CDK_Armor_CRC_Error:  return "ASCII armored damaged (CRC error)";
     case CDK_MPI_Error:        return "Invalid or missformed MPI";
     case CDK_Inv_Value:        return "Invalid parameter or value";
-    case CDK_Error_No_Key:     return "No key available";
+    case CDK_Error_No_Key:     return "No key available or not found";
     case CDK_Chksum_Error:     return "Check for key does not match";
     case CDK_Time_Conflict:    return "Time conflict";
     case CDK_Zlib_Error:       return "ZLIB error";
@@ -222,7 +222,7 @@ cdk_realloc (void *ptr, size_t size)
 }
 
 
-void *
+char *
 cdk_strdup (const char * ptr)
 {
   char * p = cdk_malloc (strlen (ptr) + 1);
@@ -397,7 +397,7 @@ cdk_handle_new (CDK_HD * r_ctx)
 
   c->digest_algo = GCRY_MD_SHA1;
   c->cipher_algo = GCRY_CIPHER_CAST5;
-  c->compress_algo = CDK_COMPRESS_ZLIB;
+  c->compress_algo = CDK_COMPRESS_ZIP;
 
   *r_ctx = c;
   return 0;
@@ -417,6 +417,12 @@ cdk_handle_set_s2k (CDK_HD hd, int mode, int digest, int cipher)
 {
   if (hd)
     {
+      if (_cdk_cipher_test_algo (cipher))
+        cipher = GCRY_CIPHER_CAST5;
+      if (_cdk_md_test_algo (digest))
+        digest = GCRY_MD_SHA1;
+      if (mode < 0 || mode > 3)
+        mode = 3;
       hd->_s2k.mode = mode;
       hd->_s2k.digest_algo = digest;
       hd->_s2k.cipher_algo = cipher;
@@ -496,6 +502,34 @@ cdk_handle_set_armor (CDK_HD hd, int val)
 }
 
 
+int
+cdk_handle_get_armor (CDK_HD hd)
+{
+  return hd->opt.armor;
+}
+
+
+/**
+ * cdk_handle_set_keydb: set the key database handle
+ * @hd: session handle
+ * @db: the database handle
+ *
+ * the function automatically detects whether this is a public or
+ * secret keyring and the right handle is set.
+ **/
+void
+cdk_handle_set_keydb (CDK_HD hd, CDK_KEYDB_HD db)
+{
+  if (hd)
+    {
+      if (db->secret)
+        hd->db.sec = db;
+      else
+        hd->db.pub = db;
+    }
+}
+
+
 /**
  * cdk_handle_set_compat: set compatibility mode for PGP.
  * @hd: session handle
@@ -523,6 +557,13 @@ cdk_handle_set_compat (CDK_HD hd, int val)
 }
 
 
+/**
+ * cdk_handle_set_callback: set the callback for filter operations
+ * @hd: the handle
+ * @cb: the callback function
+ * @cb_value: the opaque value for the function
+ *
+ **/
 void
 cdk_handle_set_callback (CDK_HD hd,
                          void (*cb) (void * opaque, int type, const char * s),
@@ -536,15 +577,20 @@ cdk_handle_set_callback (CDK_HD hd,
 }
 
 
+/**
+ * cdk_handle_free: free the main handle
+ * @hd: the handle
+ *
+ **/
 void
-cdk_handle_free (CDK_HD c)
+cdk_handle_free (CDK_HD hd)
 {
-  if (c)
+  if (hd)
     {
-      _cdk_result_verify_free (c->result.verify);
-      cdk_free (c->s2k);
-      cdk_free (c->dek);
-      cdk_free (c);
+      _cdk_result_verify_free (hd->result.verify);
+      cdk_free (hd->s2k);
+      cdk_free (hd->dek);
+      cdk_free (hd);
     }
 }
 
