@@ -63,6 +63,7 @@ int _cdk_is_idea_available (void);
 
 /*-- misc.c --*/
 u32 _cdk_timestamp (void);
+int _cdk_strcmp (const char * a, const char * b);
 u32 _cdk_buftou32 (const byte * buf);
 void _cdk_u32tobuf (u32 u, byte * buf);
 const char* _cdk_memistr (const char * buf, size_t buflen, const char * sub);
@@ -107,10 +108,15 @@ int cdk_hash_passphrase (CDK_DEK dek, char * pw, CDK_S2K_HD s2k, int creat);
 u32 _cdk_keyid_get (CDK_PACKET * pkt, u32 * keyid);
 int _cdk_fingerprint_get (CDK_PACKET * pkt, byte * fpr);
      
-CDK_KBNODE _cdk_keydb_get_pkblock (u32 * keyid);
-CDK_KBNODE _cdk_keydb_get_skblock (u32 * keyid);
+CDK_KBNODE _cdk_keydb_get_pkblock (CDK_KEYDB_HD hd, u32 * keyid);
+CDK_KBNODE _cdk_keydb_get_skblock (CDK_KEYDB_HD hd, u32 * keyid);
 CDK_KBNODE _cdk_keydb_find_bykeyid (CDK_KBNODE root, u32 * keyid);
 
+int _cdk_keydb_get_pk_byname (CDK_KEYDB_HD hd, const char * name,
+                              cdkPKT_public_key ** ret_pk, int alloced);
+int _cdk_keydb_get_sk_byname (CDK_KEYDB_HD hd, const char * name,
+                              cdkPKT_secret_key ** ret_sk, int alloced);
+
 /*-- sign.c --*/
 int _cdk_sig_create (cdkPKT_public_key * pk, cdkPKT_signature * sig);
 int _cdk_sig_hash_for (int pubkey_algo, int pkt_version);
@@ -125,6 +131,7 @@ FILE * _cdk_stream_get_fp (CDK_STREAM s);
 int _cdk_stream_gets (CDK_STREAM s, char * buf, size_t count);
 int _cdk_stream_append (const char * file, CDK_STREAM * ret_s);
 int _cdk_stream_get_errno (CDK_STREAM s);
+int _cdk_stream_set_blockmode (CDK_STREAM s, size_t nbytes);
 
 /*-- verify.c --*/
 void _cdk_result_verify_free (Verify_Result res);
@@ -133,5 +140,7 @@ Verify_Result _cdk_result_verify_new (void);
 /*-- encrypt.c --*/
 int _cdk_proc_signature_packets (CDK_HD hd, GCRY_MD_HD md,
                                  CDK_STREAM inp, CDK_STREAM out);
+/*-- read-packet.c --*/
+size_t _cdk_pkt_read_len (FILE * inp, int * ret_partial);
 
 #endif /* CDK_MAIN_H */
