@@ -39,7 +39,7 @@ _cdk_timestamp (void)
 
 
 u32
-_cdk_buftou32 (const byte *buf)
+_cdk_buftou32 (const byte * buf)
 {
   u32 u = 0;
   
@@ -55,7 +55,7 @@ _cdk_buftou32 (const byte *buf)
 
 
 void
-_cdk_u32tobuf (u32 u, byte *buf)
+_cdk_u32tobuf (u32 u, byte * buf)
 {
   if (buf)
     {
@@ -67,6 +67,19 @@ _cdk_u32tobuf (u32 u, byte *buf)
 }
 
 
+int
+_cdk_strcmp (const char * a, const char * b)
+{
+  int alen, blen;
+  
+  alen = strlen (a);
+  blen = strlen (b);
+  if (alen != blen)
+    return alen > blen? 1 : -1;
+  return strcmp (a, b);
+}
+    
+
 static const char *
 parse_version_number (const char *s, int *number)
 {
