@@ -156,13 +156,14 @@ _cdk_free_seckey (cdkPKT_secret_key * sk)
   if (sk)
     {
       nskey = cdk_pk_get_nskey (sk->pubkey_algo);
-      while (sk->mpi && nskey--)
+      while (nskey--)
         {
-          #ifndef __MINGW32__ /* in some cases, it caused a segv */
-          wipemem (sk->mpi[nskey], sk->mpi[nskey]->bytes);
-          #endif
-          cdk_free (sk->mpi[nskey]);
-          sk->mpi[nskey] = NULL; 
+          if (sk->mpi[nskey])
+            {
+              wipemem (sk->mpi[nskey], sk->mpi[nskey]->bytes);
+              cdk_free (sk->mpi[nskey]);
+              sk->mpi[nskey] = NULL;
+            }
         }
       cdk_free (sk->encdata);
       sk->encdata = NULL;
