@@ -21,7 +21,7 @@
 #ifndef OPENCDK_H
 #define OPENCDK_H
 
-#define OPENCDK_VERSION "0.3.4"
+#define OPENCDK_VERSION "0.4.0"
 
 #include <gcrypt.h>
 
@@ -94,7 +94,7 @@ typedef enum
   CDK_Wrong_Format = 22,
   CDK_Inv_Packet_Ver = 23,
   CDK_Too_Short = 24
-} cdk_err_t;
+} cdk_error_t;
 
 enum
 {
@@ -120,6 +120,14 @@ enum
   CDK_PK_ELG   = 20
 };
 
+enum
+{
+  CDK_MD_MD5    = 1,
+  CDK_MD_SHA1   = 2,
+  CDK_MD_RMD160 = 3
+};
+
+
 enum
 {
   CDK_PREFTYPE_NONE = 0,
@@ -149,11 +157,15 @@ enum
   CDK_DBSEARCH_SHORT_KEYID =   3,
   CDK_DBSEARCH_KEYID       =   4,
   CDK_DBSEARCH_FPR         =   5,
-  CDK_DBTYPE_KEYRING       = 100,
-  CDK_DBTYPE_ARMORED       = 101,
+  CDK_DBTYPE_PK_KEYRING    = 100,
+  CDK_DBTYPE_SK_KEYRING    = 101,
   CDK_DBTYPE_DATA          = 102
 };
 
+/* compatibilty */
+#define CDK_DBTYPE_ARMORED 100
+#define CDK_DBTYPE_KEYRING 100
+
 enum
 {
   CDK_KEY_VALID   = 0,
@@ -387,8 +399,8 @@ typedef struct
 
 struct cdk_packet_s
 {
-  int pktlen; /* real packet length */
-  int pktsize; /* length with all headers */
+  size_t pktlen; /* real packet length */
+  size_t pktsize; /* length with all headers */
   int old_ctb;
   cdk_packet_types_t pkttype;
   union
@@ -425,16 +437,18 @@ void * cdk_calloc (size_t n, size_t m);
 void * cdk_realloc (void * ptr, size_t size);
 void * cdk_salloc (size_t size);
 void * cdk_scalloc (size_t m);
-void * cdk_strdup (const char * ptr);
+char * cdk_strdup (const char * ptr);
 void cdk_free (void * ptr);
 
 int cdk_handle_new (CDK_HD * r_ctx);
 void cdk_handle_free (CDK_HD c);
+void cdk_handle_set_keydb (CDK_HD hd, CDK_KEYDB_HD db);
 void cdk_handle_set_s2k (CDK_HD hd, int mode, int digest, int cipher);
 void cdk_handle_set_cipher (CDK_HD hd, int cipher);
 void cdk_handle_set_digest (CDK_HD hd, int digest);
 void cdk_handle_set_compress (CDK_HD hd, int algo);
 void cdk_handle_set_armor (CDK_HD hd, int val);
+int cdk_handle_get_armor (CDK_HD hd);
 void cdk_handle_set_compat (CDK_HD hd, int val);
 void cdk_handle_set_callback (CDK_HD hd,
                               void (*cb) (void *opa, int type, const char * s),
@@ -482,18 +496,23 @@ int cdk_pk_get_npkey (int algo);
 int cdk_pk_get_nskey (int algo);
 int cdk_pk_get_nsig (int algo);
 int cdk_pk_get_nenc (int algo);
-
-/*-- verify.c --*/
-unsigned long cdk_sig_get_ulong_attr (CDK_HD hd, int idx, int what);
-const void * cdk_sig_get_data_attr (CDK_HD hd, int idx, int what);
-     
-/*-- seckey.c --*/
+int cdk_pk_get_fingerprint (cdkPKT_public_key * pk, unsigned char * fpr);
+unsigned long cdk_pk_fingerprint_get_keyid (const unsigned char * fpr,
+                                            size_t fprlen,
+                                            unsigned long * keyid);
+unsigned long cdk_pk_get_keyid (cdkPKT_public_key * pk, unsigned long * keyid);
+unsigned long cdk_sk_get_keyid (cdkPKT_secret_key * sk, unsigned long * keyid);
+unsigned long cdk_sig_get_keyid (cdkPKT_signature * sg, unsigned long * keyid);
 typedef char * (*CDK_PASSPHRASE_CB) (const char *);
 int cdk_seckey_unprotect (cdkPKT_secret_key * sk, char * pw);
 CDK_DEK cdk_passphrase_to_dek (int cipher_algo, CDK_S2K_HD s2k,
                                int mode, char * pw);
 void cdk_passphrase_cb_set (CDK_PASSPHRASE_CB pass_fnc);
 
+/*-- verify.c --*/
+unsigned long cdk_sig_get_ulong_attr (CDK_HD hd, int idx, int what);
+const void * cdk_sig_get_data_attr (CDK_HD hd, int idx, int what);
+     
 /*-- seskey.c --*/
 CDK_DEK cdk_pkcs1_to_dek (cdkPKT_pubkey_enc * enc, cdkPKT_secret_key * sk);
 int cdk_dek_new (CDK_DEK * r_dek, int algo);
@@ -535,35 +554,26 @@ int cdk_stream_kick_off (CDK_STREAM inp, CDK_STREAM out);
 
 /*-- keydb.c --*/
 int cdk_keydb_new (CDK_KEYDB_HD * r_hd, int type, void * data, size_t count);
-int cdk_keydb_open (CDK_KEYDB_HD kdb, CDK_STREAM * ret_kr);
-int cdk_keydb_add_resource (const char * name, int is_secret);
-int cdk_keydb_remove_resource (int id);
-const char * cdk_keydb_get_name (int id);
-int cdk_keydb_is_secret (int id);
-int cdk_keydb_find_idx (int is_secret, int * r_pos);
-CDK_KEYDB_HD cdk_keydb_get_ctx (int is_secret, int id);
+int cdk_keydb_open (CDK_KEYDB_HD hd, CDK_STREAM * ret_kr);
 int cdk_keydb_search_new (CDK_DBSEARCH * r_dbs, int type, void * desc);
 void cdk_keydb_search_free (CDK_DBSEARCH dbs);
-int cdk_keydb_search (CDK_KEYDB_HD kdb, CDK_DBSEARCH ks, CDK_KBNODE * r_key);
+int cdk_keydb_search (CDK_KEYDB_HD hd, CDK_DBSEARCH ks, CDK_KBNODE * r_key);
 int cdk_keydb_search_auto (CDK_KEYDB_HD hd, const char *desc,
                            CDK_KBNODE * r_knode);
 void cdk_keydb_search_reset (CDK_KEYDB_HD hd);
-int cdk_keydb_get_bykeyid (CDK_KEYDB_HD kdb, unsigned long * keyid,
+void cdk_keydb_free (CDK_KEYDB_HD hd);
+int cdk_keydb_get_bykeyid (CDK_KEYDB_HD hd, unsigned long * keyid,
                            CDK_KBNODE * r_key);
-int cdk_keydb_get_byfpr (CDK_KEYDB_HD kdb, const unsigned char * fpr,
+int cdk_keydb_get_byfpr (CDK_KEYDB_HD hd, const unsigned char * fpr,
                          CDK_KBNODE * r_key);
-int cdk_keydb_get_bypattern (CDK_KEYDB_HD kdb, const char * patt,
+int cdk_keydb_get_bypattern (CDK_KEYDB_HD hd, const char * patt,
                              CDK_KBNODE * r_pkt);
 int cdk_keydb_get_pk (CDK_KEYDB_HD khd, unsigned long * keyid,
                       cdkPKT_public_key ** ret_pk);
 int cdk_keydb_get_sk (CDK_KEYDB_HD khd, unsigned long * keyid,
                       cdkPKT_secret_key ** ret_sk);
-int cdk_keydb_get_pk_byname (const char * name, cdkPKT_public_key ** ret_pk,
-                             int alloced);
-int cdk_keydb_get_sk_byname (const char * name, cdkPKT_secret_key ** ret_sk,
-                             int alloced);
 int cdk_keydb_get_keyblock (CDK_STREAM inp, CDK_KBNODE * ret_key);
-int cdk_keydb_export (CDK_STREAM out, CDK_STRLIST remusr, int secret);
+int cdk_keydb_export (CDK_KEYDB_HD hd, CDK_STREAM out, CDK_STRLIST remusr);
 int cdk_keydb_import (CDK_KEYDB_HD hd, CDK_KBNODE knode, int result[4]);
 
 /*-- kbnode.c --*/
@@ -582,26 +592,18 @@ CDK_KBNODE cdk_kbnode_find (CDK_KBNODE node, int pkttype);
 CDK_KBNODE cdk_kbnode_find_prev (CDK_KBNODE root, CDK_KBNODE node, int pkttyp);
 CDK_KBNODE cdk_kbnode_find_next (CDK_KBNODE node, int pkttype);
 
-/*-- keyid.c --*/
-int cdk_pk_get_fingerprint (cdkPKT_public_key * pk, unsigned char * fpr);
-unsigned long cdk_pk_fingerprint_get_keyid (const unsigned char * fpr,
-                                            size_t fprlen,
-                                            unsigned long * keyid);
-unsigned long cdk_pk_get_keyid (cdkPKT_public_key * pk, unsigned long * keyid);
-unsigned long cdk_sk_get_keyid (cdkPKT_secret_key * sk, unsigned long * keyid);
-unsigned long cdk_sig_get_keyid (cdkPKT_signature * sig, unsigned long *keyid);
-
 /*-- sig-check.c --*/
-int cdk_key_check_sigs (CDK_KBNODE kb_pk, CDK_KEYDB_HD khd, int * r_status);
+int cdk_key_check_sigs (CDK_KBNODE knode, CDK_KEYDB_HD hd, int * r_status);
 
 /*-- keylist.c --*/
 int cdk_pklist_select_algo (CDK_KEYLIST pkl);
 int cdk_pklist_use_mdc (CDK_KEYLIST pkl);
-int cdk_pklist_build (CDK_STRLIST remusr, CDK_KEYLIST * ret_pkl, int use);
+int cdk_pklist_build (CDK_KEYDB_HD hd, CDK_STRLIST remusr,
+                      CDK_KEYLIST * ret_pkl, int use);
 void cdk_pklist_release (CDK_KEYLIST pkl);
 int cdk_pklist_encrypt (CDK_KEYLIST pkl, CDK_DEK dek, CDK_STREAM out);
-int cdk_sklist_build (CDK_STRLIST locusr, CDK_KEYLIST * ret_skl, int unlock,
-                      unsigned int use);
+int cdk_sklist_build (CDK_KEYDB_HD hd, CDK_STRLIST locusr,
+                      CDK_KEYLIST * ret_skl, int unlock, unsigned int use);
 void cdk_sklist_release (CDK_KEYLIST skl);
 int cdk_sklist_write (CDK_KEYLIST skl, CDK_STREAM outp, GCRY_MD_HD hash,
                       int sigclass, int sigver);
@@ -618,9 +620,7 @@ int cdk_file_decrypt (CDK_HD hd, const char * file,
                       const char * output);
 
 /*-- verify.c --*/
-int cdk_file_verify (CDK_HD hd, const char * file);
-int cdk_file_verify_clearsign (CDK_HD hd, const char * file,
-                               const char * output);
+int cdk_file_verify (CDK_HD hd, const char * file, const char * output);
 
 /*-- sign.c --*/
 int cdk_file_sign (CDK_HD hd, CDK_STRLIST locusr, CDK_STRLIST remusr,
@@ -638,7 +638,7 @@ int cdk_trustdb_get_ownertrust (CDK_STREAM inp, cdkPKT_public_key * pk,
 void cdk_strlist_free (CDK_STRLIST sl);
 CDK_STRLIST cdk_strlist_add (CDK_STRLIST * list, const char * string);
 const char * cdk_strlist_walk (CDK_STRLIST root, CDK_STRLIST * context);
-const char* cdk_check_version (const char * req_version);
+const char * cdk_check_version (const char * req_version);
 
 /* UTF8 */
 char * cdk_utf8_encode (const char * string);
@@ -646,14 +646,15 @@ char * cdk_utf8_decode (const char * string, size_t length, int delim);
 
 /*-- keyserver.c --*/
 int cdk_keyserver_recv_key (const char * host, int port,
-                            unsigned long * keyid, CDK_KBNODE * r_knode);
+                            const unsigned char * keyid, int kid_type,
+                            CDK_KBNODE * r_knode);
 
 /*-- keygen.c --*/
 int cdk_keygen_set_prefs (CDK_KEYGEN_CTX hd, int type, unsigned char * array,
                           size_t n);
 int cdk_keygen_set_algo_info (CDK_KEYGEN_CTX hd, int type, int algo, int bits);
 void cdk_keygen_set_mdc_feature (CDK_KEYGEN_CTX hd, int val);
-void cdk_keygen_set_expire_date (CDK_KEYGEN_CTX hd, long timestamp);
+void cdk_keygen_set_expire_date (CDK_KEYGEN_CTX hd, int type, long timestamp);
 void cdk_keygen_set_name (CDK_KEYGEN_CTX hd, const char * name);
 int cdk_keygen_start (CDK_KEYGEN_CTX hd);
 int cdk_keygen_save (CDK_KEYGEN_CTX hd, const char * pub, const char * sec);
