@@ -34,8 +34,8 @@ typedef struct
   int ctb;
   int type;
   unsigned rfc2440:1;
-  int len;
-  int size;
+  size_t len;
+  size_t size;
   unsigned partial:1;
 } PKTHEAD;
 
@@ -60,7 +60,7 @@ stream_read (CDK_STREAM s, void * buf, size_t count, size_t * r_nread)
 }
 
 
-u32
+static u32
 read_32 (CDK_STREAM buf, int * ret_err)
 {
   u32 u = 0;
@@ -86,7 +86,7 @@ read_32 (CDK_STREAM buf, int * ret_err)
 }
 
 
-u16
+static u16
 read_16 (CDK_STREAM buf, int * ret_err)
 {
   u16 u = 0;
@@ -181,31 +181,33 @@ read_mpi (CDK_STREAM inp, CDK_MPI * ret_m, int secure)
 }
 
 
-static int
-read_pkt_length (CDK_STREAM inp, int * ret_partial)
+size_t
+_cdk_pkt_read_len (FILE * inp, int * ret_partial)
 {
   int c1 = 0, c2 = 0;
   size_t pktlen = 0;
-  int rc = 0;
 
-  if ((c1 = stream_getc (inp)) == EOF)
-    return EOF;
+  if ((c1 = fgetc (inp)) == EOF)
+    return (size_t)EOF;
   if (c1 < 224 || c1 == 255)
     *ret_partial = 0; /* end of partial data */
   if (c1 < 192)
     pktlen = c1;
   else if (c1 >= 192 && c1 <= 223)
     {
-      c2 = stream_getc (inp);
+      c2 = fgetc (inp);
       if (c2 == EOF)
-	return EOF;
+	return (size_t)EOF;
       pktlen = ((c1 - 192) << 8) + c2 + 192;
     }
   else if (c1 == 255)
     {
-      pktlen = read_32 (inp, &rc);
-      if (rc)
-	return EOF;
+      pktlen  = fgetc (inp) << 24;
+      pktlen |= fgetc (inp) << 16;
+      pktlen |= fgetc (inp) <<  8;
+      pktlen |= fgetc (inp);
+      if (!pktlen)
+	return (size_t)EOF;
     }
   else
     pktlen = 1 << (c1 & 0x1f);
@@ -217,8 +219,6 @@ static int
 read_encrypted (CDK_STREAM inp, size_t pktlen, cdkPKT_encrypted * enc,
 		int partial, int mdc)
 {
-  byte buf[8192];
-  size_t nread = 0, buflen;
   int rc = 0, version;
 
   if (!inp || !enc)
@@ -227,10 +227,6 @@ read_encrypted (CDK_STREAM inp, size_t pktlen, cdkPKT_encrypted * enc,
   if (DEBUG_PKT)
     _cdk_log_debug ("** read encrypted packet %d bytes\n", pktlen);
 
-  enc->buf = cdk_stream_tmp ();
-  if (!enc->buf)
-    return CDK_Out_Of_Core;
-
   if (mdc)
     {
       version = stream_getc (inp);
@@ -239,28 +235,14 @@ read_encrypted (CDK_STREAM inp, size_t pktlen, cdkPKT_encrypted * enc,
       enc->mdc_method = GCRY_MD_SHA1;
       pktlen--;
     }
-  while (!cdk_stream_eof (inp))
-    {
-      buflen = partial? pktlen : sizeof buf-1;
-      rc = stream_read (inp, buf, buflen, &nread);
-      if (rc)
-        goto leave;
-      pktlen -= nread;
-      cdk_stream_write (enc->buf, buf, nread);
-      if (partial)
-	{
-	  pktlen = read_pkt_length (inp, &partial);
-	  if (pktlen == EOF)
-	    return CDK_Inv_Packet;
-	}
-    }
-  wipemem (buf, sizeof buf);
-  if (pktlen)
-    return CDK_Inv_Packet;
-  cdk_stream_seek (enc->buf, 0);
-  enc->len = cdk_stream_get_length (enc->buf);
+  if (pktlen < 10)
+    return CDK_Inv_Packet; /* we need at least blocksize + 2 bytes */
+
+  if (partial)
+    _cdk_stream_set_blockmode (inp, pktlen);
+  enc->buf = inp;
+  enc->len = cdk_stream_get_length (enc->buf) - cdk_stream_tell (inp);
 
-leave:
   return rc;
 }
 
@@ -386,8 +368,11 @@ read_compressed (CDK_STREAM inp, size_t pktlen, cdkPKT_compressed * c)
     return CDK_Inv_Packet;
 
   /* don't know the size, so we read until EOF */
-  c->len = 0;
-  c->buf = inp;
+  if (!pktlen)
+    {
+      c->len = 0;
+      c->buf = inp;
+    }
 
   return 0;
 }
@@ -554,8 +539,8 @@ static int
 read_attribute (CDK_STREAM inp, size_t pktlen, cdkPKT_user_id * attr)
 {
   size_t nread = 0;
-  byte *buf;
-  const byte *p;
+  byte * buf;
+  const byte * p;
   int len = 0;
   int rc = 0;
 
@@ -613,9 +598,11 @@ read_user_id (CDK_STREAM inp, size_t pktlen, cdkPKT_user_id * user_id)
   size_t nread = 0;
   int rc = 0;
 
-  if (!inp || !user_id || !pktlen)
+  if (!inp || !user_id)
     return CDK_Inv_Value;
-
+  if (!pktlen)
+    return CDK_Inv_Packet;
+  
   if (DEBUG_PKT)
     _cdk_log_debug ("** read user ID packet\n");
 
@@ -872,17 +859,17 @@ read_old_length (PKTHEAD * ph, CDK_STREAM inp)
   if ((ph->ctb & 0x03) == 0)
     {
       ph->len = stream_getc (inp);
-      ph->size++; 
+      ph->size++;
     }
   else if ((ph->ctb & 0x03) == 1)
     {
       ph->len = read_16 (inp, NULL);
-      ph->size += 2; 
+      ph->size += 2;
     }
   else if ((ph->ctb & 0x03) == 2)
     {
       ph->len = read_32 (inp, NULL);
-      ph->size += 4; 
+      ph->size += 4;
     }
   else
     {
@@ -905,7 +892,7 @@ read_new_length (PKTHEAD * ph, CDK_STREAM inp)
     {
       c1 = stream_getc (inp);
       ph->size++;
-      ph->len = ((c - 192) << 8) + c1 + 192; 
+      ph->len = ((c - 192) << 8) + c1 + 192;
     }
   else if (c == 255)
     {
@@ -920,11 +907,20 @@ read_new_length (PKTHEAD * ph, CDK_STREAM inp)
 }
 
 
+/* we use a buffer to make it faster to skip larger unknown packets. */
 static void
-skip_packet (CDK_STREAM inp, int pktsize)
+skip_packet (CDK_STREAM inp, size_t pktlen)
 {
-  while (pktsize--)
-    stream_getc (inp);  
+  byte buf[4096];
+  size_t nread;
+  
+  while (pktlen > 4095)
+    {
+      stream_read (inp, buf, sizeof buf-1, &nread);
+      pktlen -= nread;
+    }
+  while (pktlen--)
+    stream_getc (inp);
 }
 
 
