@@ -98,7 +98,7 @@ _cdk_hash_pubkey (cdkPKT_public_key * pk, GCRY_MD_HD md, int use_fpr)
 void
 _cdk_hash_userid (cdkPKT_user_id * uid, int sig_version, GCRY_MD_HD md)
 {
-  const byte *data;
+  const byte * data;
   byte buf[5];
   u32 dlen;
 
@@ -243,8 +243,8 @@ _cdk_signature_check (cdkPKT_public_key * pk, cdkPKT_signature * sig,
 static int
 check_key_signature (CDK_KEYDB_HD hd, CDK_KBNODE knode, CDK_KBNODE snode)
 {
-  cdkPKT_public_key *pk = NULL, *sig_pk = NULL;
-  cdkPKT_signature *sig = NULL;
+  cdkPKT_public_key * pk = NULL, * sig_pk = NULL;
+  cdkPKT_signature * sig = NULL;
   CDK_KBNODE node;
   GCRY_MD_HD md;
   int digest_algo, is_expired = 0;
@@ -303,7 +303,6 @@ check_key_signature (CDK_KEYDB_HD hd, CDK_KBNODE knode, CDK_KBNODE snode)
 	rc = _cdk_signature_check (pk, sig, md, &is_expired);
       else if (hd)
 	{
-          cdk_keydb_search_reset (hd);
 	  rc = cdk_keydb_get_pk (hd, sig->keyid, &sig_pk);
 	  if (rc)
 	    return CDK_Error_No_Key;
