@@ -36,19 +36,16 @@ static void
 sig_calc_subpkt_size (cdkPKT_signature * sig)
 {
   size_t nbytes;
-  byte * p;
   
   if (sig->hashed)
     {
-      p = _cdk_subpkt_get_array (sig->hashed, 1, &nbytes);
+      _cdk_subpkt_get_array (sig->hashed, 1, &nbytes);
       sig->hashed_size = nbytes;
-      cdk_free (p);
     }
   if (sig->unhashed)
     {
-      p = _cdk_subpkt_get_array (sig->unhashed, 1, &nbytes);
+      _cdk_subpkt_get_array (sig->unhashed, 1, &nbytes);
       sig->unhashed_size = nbytes;
-      cdk_free (p);
     }  
 }
 
@@ -133,6 +130,9 @@ _cdk_sig_complete (cdkPKT_signature * sig, cdkPKT_secret_key * sk,
 {
   byte digest[24];
 
+  if (!sig || !sk)
+    return CDK_Inv_Value;
+  
   sig_calc_subpkt_size (sig);
   _cdk_hash_sig_data (sig, md);
   gcry_md_final (md);
@@ -151,6 +151,9 @@ write_plaintext (CDK_STREAM inp, CDK_STREAM out)
   const char * s = _cdk_stream_get_fname (inp);
   int rc;
 
+  if (!inp || !out)
+    return CDK_Inv_Value;
+  
   cdk_stream_seek (inp, 0);
   pt = cdk_calloc (1, sizeof *pt + strlen (s));
   if (!pt)
@@ -176,7 +179,10 @@ write_pubkey_enc_list (CDK_HD hd, CDK_STREAM out, CDK_STRLIST remusr)
   CDK_KEYLIST pkl;
   int rc;
 
-  rc = cdk_pklist_build (remusr, &pkl, GCRY_PK_USAGE_ENCR);
+  if (!out)
+    return CDK_Inv_Value;
+  
+  rc = cdk_pklist_build (hd->db.pub, remusr, &pkl, GCRY_PK_USAGE_ENCR);
   if (rc)
     return rc;
 
@@ -200,6 +206,7 @@ get_sig_version (CDK_HD hd, CDK_KEYLIST kl)
 
   if (hd->opt.compat)
     return 3;
+  
   for (l = kl; l; l = l->next)
     {
       if ((l->type == CDK_PKT_PUBLIC_KEY && l->key.pk->version == 3)
@@ -216,7 +223,7 @@ get_sig_version (CDK_HD hd, CDK_KEYLIST kl)
  * @remusr: If encrypt is valid, the list of recipients
  * @file: Name of the input file
  * @output: Name of the output file
- * @detached: Create a detached signature (?)
+ * @detached: Create a detached signature (1=yes, 0=no)
  * @encrypt: enable sign and encrypt
  *
  **/
@@ -250,7 +257,7 @@ cdk_file_sign (CDK_HD hd, CDK_STRLIST locusr, CDK_STRLIST remusr,
       return rc;
     }
 
-  rc = cdk_sklist_build (locusr, &list, 1, GCRY_PK_USAGE_SIGN);
+  rc = cdk_sklist_build (hd->db.sec, locusr, &list, 1, GCRY_PK_USAGE_SIGN);
   if (rc)
     {
       cdk_stream_close (inp);
@@ -370,7 +377,7 @@ cdk_file_clearsign (CDK_HD hd, CDK_STRLIST locusr,
       return rc;
     }
 
-  rc = cdk_sklist_build (locusr, &list, 1, GCRY_PK_USAGE_SIGN);
+  rc = cdk_sklist_build (hd->db.sec, locusr, &list, 1, GCRY_PK_USAGE_SIGN);
   if (rc)
     {
       cdk_stream_close (inp);
