@@ -14,8 +14,8 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with OpenCDK; if not, write to the Cdk_Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
+ * along with OpenCDK; if not, write to the Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA
  */
 
 #ifdef HAVE_CONFIG_H
@@ -75,11 +75,12 @@ mkstemp (char * template)
 static char *
 mk_tmpnam (void)
 {
-  static char template[11];
-  
+  static char template[14];
+
+  /* fixme: add W32 code */
   memset (template, 0, sizeof template);
-  strncpy (template, "cdk", 3);
-  memset (template + 3, 'X', 6);
+  strncpy (template, "/tmp/cdk", 8);
+  memset (template + 8, 'X', 6);
   mkstemp (template);
   return template;
 }
@@ -289,6 +290,8 @@ cdk_stream_flush (CDK_STREAM s)
 
   if (!s->flags.filtrated)
     {
+      if (!cdk_stream_get_length (s))
+        return 0;
       rc = fseek (s->fp, 0, SEEK_SET);
       if (!rc)
         rc = fflush (s->fp);
@@ -854,6 +857,11 @@ cdk_stream_set_cipher_flag (CDK_STREAM s, CDK_DEK dek, int use_mdc)
   dek->use_mdc = use_mdc;
   flt->u.cfx.dek = dek;
   flt->u.cfx.mdc_method = use_mdc? GCRY_MD_SHA1 : 0;
+  if (s->blkmode)
+    {
+      flt->u.cfx.blkmode.on = 1;
+      flt->u.cfx.blkmode.size = s->blkmode;
+    }
   return 0;
 }
 
@@ -943,7 +951,7 @@ stream_cache_flush (CDK_STREAM s, FILE * fp)
 int
 cdk_stream_kick_off (CDK_STREAM inp, CDK_STREAM out)
 {
-  byte buf[4096];
+  byte buf[8192];
   int nread, nwritten;
   int rc = 0;
 
@@ -981,3 +989,16 @@ _cdk_stream_gets (CDK_STREAM s, char * buf, size_t count)
 }
 
 
+int
+_cdk_stream_set_blockmode (CDK_STREAM s, size_t nbytes)
+{
+  if (!s)
+    return CDK_Inv_Value;
+  
+  _cdk_log_debug ("set block mode for stream (size=%d)\n", nbytes);
+  
+  s->blkmode = nbytes;
+  
+  return 0;
+}
+
