@@ -48,15 +48,31 @@ struct
 };
 
 
+static int file_verify_clearsign (CDK_HD hd, const char *, const char *);
+
+
 int
-cdk_file_verify (CDK_HD hd, const char *file)
+cdk_file_verify (CDK_HD hd, const char * file, const char * output)
 {
   CDK_STREAM inp;
   int rc;
 
+  if (!hd || !file)
+    return CDK_Inv_Value;
+  
   rc = cdk_stream_open (file, &inp);
   if (rc)
     return rc;
+  if (cdk_armor_filter_use (inp))
+    {
+      /* fixme: check for one-pass sigs? */
+      rc = file_verify_clearsign (hd, file, output);
+      if (!rc)
+        {
+          cdk_stream_close (inp);
+          return rc;
+        }
+    }
   rc = _cdk_proc_signature_packets (hd, NULL, inp, NULL);
   cdk_stream_close (inp);
   
@@ -149,8 +165,8 @@ stream_peek (CDK_STREAM inp, char *s, size_t count)
 }
 
 
-int
-cdk_file_verify_clearsign (CDK_HD hd, const char * file, const char * output)
+static int
+file_verify_clearsign (CDK_HD hd, const char * file, const char * output)
 {
   CDK_STREAM inp = NULL, out = NULL, tmp = NULL;
   GCRY_MD_HD md = NULL;
