@@ -239,7 +239,7 @@ pkt_write_head (CDK_STREAM out, int old_ctb, size_t size, int type)
 static int
 write_encrypted (CDK_STREAM out, cdkPKT_encrypted * enc, int old_ctb)
 {
-  int nbytes = 0;
+  size_t nbytes;
   int rc = 0;
 
   if (!out || !enc)
@@ -259,8 +259,8 @@ write_encrypted (CDK_STREAM out, cdkPKT_encrypted * enc, int old_ctb)
 static int
 write_encrypted_mdc (CDK_STREAM out, cdkPKT_encrypted * enc)
 {
+  size_t nbytes;
   int rc = 0;
-  int nbytes = 0;
 
   if (!out || !enc)
     return CDK_Inv_Value;
@@ -321,8 +321,8 @@ write_symkey_enc (CDK_STREAM out, cdkPKT_symkey_enc * ske)
 static int
 write_pubkey_enc (CDK_STREAM out, cdkPKT_pubkey_enc * pke, int old_ctb)
 {
-  int size, nenc = 0;
-  int rc = 0;
+  size_t size;
+  int rc = 0, nenc = 0;
 
   if (!out || !pke)
     return CDK_Inv_Value;
@@ -387,9 +387,8 @@ static int
 write_signature (CDK_STREAM out, cdkPKT_signature * sig, int old_ctb)
 {
   byte * buf;
-  size_t nbytes;
-  int size, nsig = 0;
-  int rc = 0;
+  size_t nbytes, size;
+  int nsig = 0, rc = 0;
 
   if (!out || !sig)
     return CDK_Inv_Value;
@@ -481,8 +480,8 @@ write_public_key (CDK_STREAM out, cdkPKT_public_key * pk,
                   int is_subkey, int old_ctb)
 {
   int rc = 0;
-  int size = 6, pkttype, ndays = 0;
-  size_t npkey = 0;
+  int pkttype, ndays = 0;
+  size_t npkey = 0, size = 6;
 
   if (!out || !pk)
     return CDK_Inv_Value;
@@ -649,7 +648,7 @@ write_compressed (CDK_STREAM out, cdkPKT_compressed * cd)
 static int
 write_plaintext (CDK_STREAM out, cdkPKT_plaintext * pt, int old_ctb)
 {
-  byte buf[4096];
+  byte buf[8192];
   size_t size;
   int rc;
 
