@@ -159,7 +159,7 @@ callback (void * opaque, int type, const char * s)
 static void
 progr_callback (void * opaque, unsigned off, unsigned len)
 {
-  printf ("off=%d len=%d\n", off, len);
+  printf ("** off=%d len=%d\n", off, len);
 }
 
 
@@ -221,7 +221,7 @@ key_get_letter (int pubkey_algo)
 
 
 static const char *
-key_get_date (long timestamp)
+strdate_from_long (long timestamp)
 {
   static char kdate[32];
   struct tm * tm;
@@ -272,10 +272,7 @@ algo_get_id (const char *algo, int what)
       if (!strcmp (algo, algo_table[i].name))
 	return algo_table[i].algid;
     }
-  if (what)
-    i = GCRY_CIPHER_3DES;
-  else
-    i = GCRY_MD_SHA1;
+  i = what? GCRY_CIPHER_3DES: GCRY_MD_SHA1;
   return i;
 }
 
@@ -317,12 +314,12 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
 		   cdk_pk_get_nbits (key),
 		   key_get_letter (key->pubkey_algo),
 		   (unsigned long)cdk_pk_get_keyid (key, NULL),
-		   key_get_date (key->timestamp));
+		   strdate_from_long (key->timestamp));
           if (!key->expiredate)
             fputc ('\n', stdout);
           else
             fprintf (stdout, "  [expires: %s]\n",
-                     key_get_date (key->expiredate));
+                     strdate_from_long (key->expiredate));
 	}
       else if (pkt->pkttype == CDK_PKT_PUBLIC_SUBKEY)
 	{
@@ -332,12 +329,12 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
 		   cdk_pk_get_nbits (key),
 		   key_get_letter (key->pubkey_algo),
 		   (unsigned long)cdk_pk_get_keyid (key, NULL),
-		   key_get_date (key->timestamp));
+		   strdate_from_long (key->timestamp));
           if (!key->expiredate)
             fputc ('\n', stdout);
           else
             fprintf (stdout, "  [expires: %s]\n",
-                     key_get_date (key->expiredate));
+                     strdate_from_long (key->expiredate));
 	}
       else if (pkt->pkttype == CDK_PKT_USER_ID)
 	{
@@ -354,30 +351,41 @@ do_show_key (CDK_KBNODE pk, int with_sigs)
 		   sig_get_letter (sig),
 		   sig_get_status (sig),
 		   (unsigned long)cdk_sig_get_keyid (sig, NULL),
-		   key_get_date (sig->timestamp));
+		   strdate_from_long (sig->timestamp));
 	}
     }
 }
 
 
-int
-do_list_keys (CDK_STRLIST keys)
+static int
+check_for_wildcards (CDK_STRLIST list)
 {
-  const char * ring, * s;
+  CDK_STRLIST ctx = NULL;
+  const char * s = cdk_strlist_walk (list, &ctx);
+  if (*s == '*')
+    return 1;
+  return 0;
+}
+
+
+int
+do_list_keys (CDK_KEYDB_HD hd, CDK_STRLIST keys)
+{
+  const char * s;
   unsigned char fpr[20];
   CDK_PACKET * pkt;
   CDK_STREAM inp;
   CDK_KBNODE pk = NULL;
-  CDK_KEYDB_HD khd = NULL;
   CDK_STRLIST ctx = NULL;
   int rc = 0, i;
 
+  if (check_for_wildcards (keys))
+    keys = NULL;
   if (keys)
     {
-      khd = cdk_keydb_get_ctx (0, -1);
       while ((s = cdk_strlist_walk (keys, &ctx)))
         {
-	  rc = cdk_keydb_get_bypattern (khd, s, &pk);
+	  rc = cdk_keydb_get_bypattern (hd, s, &pk);
 	  if (rc)
 	    fprintf (stdout, "get key `%s': %s\n", s, cdk_strerror (rc));
 	  if (!rc)
@@ -397,16 +405,10 @@ do_list_keys (CDK_STRLIST keys)
       return 0;
     }
   /* Show all keys */
-  ring = cdk_keydb_get_name (-1);
-  if (!ring)
-    {
-      fprintf (stderr, "No keyring was found.\n");
-      return CDK_General_Error;
-    }
-  rc = cdk_stream_open (ring, &inp);
+  rc = cdk_keydb_open (hd, &inp);
   if (rc)
     {
-      fprintf (stderr, "`%s': %s\n", ring, cdk_strerror (rc));
+      fprintf (stderr, "keydb open: %s\n", cdk_strerror (rc));
       return rc;
     }
 
@@ -426,7 +428,7 @@ do_list_keys (CDK_STRLIST keys)
 
 
 int
-do_export_keys (CDK_STRLIST keys, int secret)
+do_export_keys (CDK_HD ctx, CDK_KEYDB_HD hd, CDK_STRLIST keys, int secret)
 {
   CDK_STREAM outp = NULL;
   int rc = 0;
@@ -445,7 +447,9 @@ do_export_keys (CDK_STRLIST keys, int secret)
     }
   if (keys)
     {
-      rc = cdk_keydb_export (outp, keys, 0);
+      if (cdk_handle_get_armor (ctx))
+        cdk_stream_set_armor_flag (outp, CDK_ARMOR_PUBKEY);
+      rc = cdk_keydb_export (hd, outp, keys);
       if (rc)
 	fprintf (stdout, "key export `%s'\n", cdk_strerror (rc));
     }
@@ -455,14 +459,12 @@ do_export_keys (CDK_STRLIST keys, int secret)
 
 
 int
-do_import_keys (const char * keyfile)
+do_import_keys (CDK_KEYDB_HD hd, const char * keyfile)
 {
-  CDK_KEYDB_HD hd;
   CDK_KBNODE node = NULL, n, ctx = NULL;
   CDK_STREAM s;
   int rc, res[4];
 
-  hd = cdk_keydb_get_ctx (0, -1);
   if (!hd)
     {
       fprintf (stderr, "No public keyring available.\n");
@@ -494,8 +496,11 @@ do_import_keys (const char * keyfile)
               printf ("public keys read/updated: %d/%d\n"
                       "secret keys read/updated: %d/%d\n",
                       res[0], res[2], res[1], res[3]);
-              printf ("`%s' successfully imported.\n",
-                      pkt->pkt.user_id->name);
+              if (res[2] || res[3])
+                printf ("`%s' successfully imported.\n",
+                        pkt->pkt.user_id->name);
+              else
+                printf ("nothing changed.\n");
               break;
             }
         }
@@ -507,9 +512,8 @@ do_import_keys (const char * keyfile)
   
 
 int
-do_list_sigs (CDK_STRLIST keys)
+do_list_sigs (CDK_KEYDB_HD hd, CDK_STRLIST keys)
 {
-  CDK_KEYDB_HD hd = NULL;
   CDK_KBNODE pk = NULL;
   CDK_STRLIST ctx = NULL;
   const char * s;
@@ -518,7 +522,6 @@ do_list_sigs (CDK_STRLIST keys)
 
   if (keys)
     {
-      hd = cdk_keydb_get_ctx (0, -1);
       while ((s = cdk_strlist_walk (keys, &ctx)))
 	{
 	  rc = cdk_keydb_get_bypattern (hd, s, &pk);
@@ -557,6 +560,7 @@ int
 ks_fetch_key (const char * kid)
 {
   unsigned long keyid;
+  unsigned char kidbuf[4];
   CDK_KBNODE pk = NULL;
   CDK_PACKET * pkt;
   int rc;
@@ -573,9 +577,13 @@ ks_fetch_key (const char * kid)
   else
     keyid = strtoul (kid, NULL, 16);
 
+  kidbuf[0] = keyid >> 24;
+  kidbuf[1] = keyid >> 16;
+  kidbuf[2] = keyid >>  8;
+  kidbuf[3] = keyid;
   printf ("fetch %08lX from the keyserver...", keyid);
   rc = cdk_keyserver_recv_key ("http://horowitz.surfnet.nl", 11371,
-                               &keyid, &pk);
+                               kidbuf, CDK_DBSEARCH_SHORT_KEYID, &pk);
   if (rc)
       printf ("%s\n", cdk_strerror (rc));
   else
@@ -674,7 +682,7 @@ genkey (void)
   cdk_keygen_set_algo_info (hd, 1, GCRY_PK_ELG_E, 768);
   cdk_keygen_set_name (hd, "Otto Bismarck");
   cdk_keygen_set_mdc_feature (hd, 1);
-  cdk_keygen_set_expire_date (hd, 1040820537);
+  cdk_keygen_set_expire_date (hd, 0, 1040820537);
   rc = cdk_keygen_start (hd);
   if (!rc)
     rc = cdk_keygen_save (hd, "pub.test", "sec.test");
@@ -683,29 +691,45 @@ genkey (void)
 }
 
 
+void
+file_error (const char * fil, const char * fnc, int rc)
+{
+  fprintf (stderr, "minpg: %s %s: `%s'\n", fil, fnc, cdk_strerror (rc));
+}
+
+
 int
 main (int argc, char ** argv)
 {
   CDK_HD hd;
   CDK_STRLIST remusr = NULL, klist = NULL;
+  CDK_KEYDB_HD sec_db, pub_db;
   poptContext opt_ctx;
-  char * output = NULL;
+  char * output = NULL, * s;
   char u[128];
   int c = 0, verbose = 0, i = 0;
   int rc = 0;
 
-  cdk_keydb_add_resource ("/home/twoaday/.gnupg/pubring.cdk", 0);
-  /*cdk_keydb_add_resource ("pubring.gpg", 0);*/
-  cdk_keydb_add_resource ("sec-with-pwd.gpg", 1);
-  /*cdk_keydb_add_resource ("secring.gpg", 1);*/
-  /*cdk_keydb_add_resource ("sec2.gpg", 1);*/
+  s = "/home/twoaday/.gnupg/pubring.cdk";
+  rc = cdk_keydb_new (&pub_db, CDK_DBTYPE_PK_KEYRING, s, strlen (s));
+  if (rc)
+    {
+      fprintf (stderr, "keydb new: `%s'\n", cdk_strerror (rc));
+      return 0;
+    }
+  s = "sec-with-pwd.gpg";
+  rc = cdk_keydb_new (&sec_db, CDK_DBTYPE_SK_KEYRING, s, strlen (s));
+  if (rc)
+    {
+      fprintf (stderr, "keydb new: `%s'\n", cdk_strerror (rc));
+      return 0;
+    }
+  
   /*cdk_passphrase_cb_set (get_passphrase);*/
-  /*cdk_keydb_add_resource ("/home/twoaday/.pgp/pubring.pgp", 0);
-    cdk_keydb_add_resource ("/home/twoaday/.pgp/secring.pgp", 1);*/
 
   cdk_handle_new (&hd);
   cdk_handle_set_callback (hd, callback, NULL);
-  cdk_set_progress_handler (progr_callback, NULL);
+  /*cdk_set_progress_handler (progr_callback, NULL);*/
   opt_ctx = poptGetContext ("pred", argc, argv, arguments, 0);
   while ((c = poptGetNextOpt (opt_ctx)) >= 0)
     {
@@ -762,16 +786,17 @@ main (int argc, char ** argv)
 	  break;
 
         case 'y':
-          rc = cdk_file_verify_clearsign (hd, file, NULL);
+          cdk_handle_set_keydb (hd, pub_db);
+          rc = cdk_file_verify (hd, file, NULL);
           if (rc)
-            fprintf (stderr, "verify `%s' failed: %s\n", file,
-                     cdk_strerror (rc));
+            file_error ("verify", file, rc);
           else
             print_verify_stats (hd);
           break;
           
 	case 'u':
-	  output = cdk_calloc (1, strlen (file));
+          cdk_set_log_level (CDK_LOG_DEBUG);
+	  output = cdk_calloc (1, strlen (file) + 1);
 	  if (strstr (file, ".asc"))
 	    strncpy (output, file, strlen (file) - 4);
 	  else
@@ -780,13 +805,13 @@ main (int argc, char ** argv)
 	    fprintf (stdout, "dearmor `%s' -> %s\n", file, output);
 	  rc = cdk_file_dearmor (file, output);
 	  if (rc)
-	    fprintf (stderr, "minpg: dearmor file `%s': %s\n",
-		     file, cdk_strerror (rc));
+            file_error ("dearmor file", file, rc);
 	  break;
 
 	case 'l':
+          cdk_set_log_level (CDK_LOG_DEBUG);
 	  cdk_strlist_add (&klist, user);
-	  do_list_sigs (klist);
+	  do_list_sigs (pub_db, klist);
 	  break;
 
 	case 'a':
@@ -795,8 +820,7 @@ main (int argc, char ** argv)
 	    fprintf (stdout, "armor `%s' -> %s\n", file, output);
 	  rc = cdk_file_armor (hd, file, output);
 	  if (rc)
-	    fprintf (stdout, "minpg: armor file `%s': %s\n", file,
-		     cdk_strerror (rc));
+            file_error ("armor file", file, rc);
 	  break;
 
 	case 'z':
@@ -814,24 +838,24 @@ main (int argc, char ** argv)
               cdk_handle_set_digest (hd, digest_algid);
               cdk_handle_set_s2k (hd, 3, digest_algid, cipher_algid);
             }
+          cdk_handle_set_keydb (hd, pub_db);
 	  rc = cdk_file_encrypt (hd, NULL, file, output);
 	  if (rc)
-	    fprintf (stdout, "minpg: symmetric file `%s': %s\n",
-		     file, cdk_strerror (rc));
+            file_error ("symmetric file", file, rc);
 	  break;
 
 	case 'k':
 	  cdk_strlist_add (&klist, user);
-	  do_list_keys (klist);
+	  do_list_keys (pub_db, klist);
 	  break;
 
 	case 'x':
 	  cdk_strlist_add (&klist, user);
-	  do_export_keys (klist, 0);
+	  do_export_keys (hd, pub_db, klist, 0);
 	  break;
 
         case 'i':
-          do_import_keys (file);
+          do_import_keys (pub_db, file);
           break;
 
         case 'f':
@@ -854,10 +878,10 @@ main (int argc, char ** argv)
 	  output = do_add_ext (file, ".gpg");
 	  if (verbose)
 	    fprintf (stdout, "encrypt `%s' -> %s\n", file, output);
+          cdk_handle_set_keydb (hd, pub_db);
 	  rc = cdk_file_encrypt (hd, remusr, file, output);
 	  if (rc)
-	    fprintf (stdout, "minpg: encrypt file `%s': %s\n",
-		     file, cdk_strerror (rc));
+            file_error ("encrypt file", file, rc);
 	  break;
 
 	case 'd':
@@ -865,10 +889,11 @@ main (int argc, char ** argv)
 	  output = do_remove_ext (file);
 	  if (verbose)
 	    fprintf (stdout, "decrypt `%s' -> %s\n", file, output);
+          cdk_handle_set_keydb (hd, sec_db);
+          cdk_handle_set_keydb (hd, pub_db);
 	  rc = cdk_file_decrypt (hd, file, output);
 	  if (rc)
-	    fprintf (stdout, "minpg: decrypt file `%s' failed: %s\n",
-		     file, cdk_strerror (rc));
+            file_error ("decrypt file", file, rc);
 	  break;
 
         case 'w':
@@ -886,10 +911,10 @@ main (int argc, char ** argv)
           output = do_add_ext (file, ".sig");
           if (verbose)
             fprintf (stdout, "sign `%s' -> %s\n", file, output);
+          cdk_handle_set_keydb (hd, sec_db);
           rc = cdk_file_sign (hd, remusr, NULL, file, output, 1, 0);
           if (rc)
-            fprintf (stderr, "minpg: sign file `%s' failed: %s\n",
-                     file, cdk_strerror (rc));
+            file_error ("sign file", file, rc);
           break;
 
         case 't':
@@ -899,10 +924,10 @@ main (int argc, char ** argv)
               exit (0);
             }
           output = do_add_ext (file, ".asc");
+          cdk_handle_set_keydb (hd, sec_db);
           rc = cdk_file_clearsign (hd, remusr, file, output);
           if (rc)
-            fprintf (stderr, "minpg: clearsign file `%s': failed: %s\n",
-                     file, cdk_strerror (rc));
+            file_error ("clearsign file", file, rc);
           break;
 
         case 'b':
@@ -913,14 +938,17 @@ main (int argc, char ** argv)
             }
           output = do_add_ext (file, ".gpg");
           cdk_strlist_add (&klist, "opencdk");
+          cdk_handle_set_keydb (hd, pub_db);
+          cdk_handle_set_keydb (hd, sec_db);
           rc = cdk_file_sign (hd, klist, remusr, file, output, 0, 1);
           if (rc)
-            fprintf (stderr, "minpg: sign and encrypt `%s' failed: %s\n",
-                     file, cdk_strerror (rc));
+            file_error ("sign encrypt", file, rc);
           break;
 	}
     }
 
+  cdk_keydb_free (pub_db);
+  cdk_keydb_free (sec_db);
   cdk_handle_free (hd);
   cdk_strlist_free (remusr);
   cdk_free (output);
