@@ -221,11 +221,13 @@ stream_encrypt_pk (char *user, char *file)
   int rc;
   CDK_HD hd;
   CDK_STRLIST recp = NULL;
-
+  CDK_KEYDB_HD db;
+  
   cdk_handle_new (&hd);
-  cdk_keydb_add_resource ("ts.gpg", 0);
+  cdk_keydb_new (&db, CDK_DBTYPE_PK_KEYRING, "ts.gpg", 6);
   cdk_strlist_add (&recp, user);
 
+  cdk_handle_set_keydb (hd, db);
   rc = cdk_file_encrypt (hd, recp, file, "out.gpg");
   if (rc)
     printf ("encrypt_file %s: `%s'\n", file, cdk_strerror (rc));
@@ -313,13 +315,16 @@ stream_decrypt (char *file, int with_pass)
   int rc, sigstat, created, pkalgo;
   const unsigned long * keyid;
   CDK_HD hd;
+  CDK_KEYDB_HD db, p_db;
 
   if (!with_pass)
-    cdk_keydb_add_resource ("sec.gpg", 1);
+    cdk_keydb_new (&db, CDK_DBTYPE_SK_KEYRING, "sec.gpg", 7);
   else
-    cdk_keydb_add_resource ("sec-with-pwd.gpg", 1);
-  cdk_keydb_add_resource ("pub.gpg", 0);
+    cdk_keydb_new (&db, CDK_DBTYPE_SK_KEYRING, "sec-with-pwd.gpg", 16);
+  cdk_keydb_new(&p_db, CDK_DBTYPE_PK_KEYRING, "pub.gpg", 7);
   cdk_handle_new (&hd);
+  cdk_handle_set_keydb (hd, db);
+  cdk_handle_set_keydb (hd, p_db);
   rc = cdk_file_decrypt (hd, file, "out.gpg");
   if (rc)
     printf ("decrypt file %s: `%s'\n", file, cdk_strerror (rc));
@@ -414,11 +419,13 @@ static int
 stream_verify (char *file)
 {
   CDK_HD hd;
+  CDK_KEYDB_HD db;
   int rc;
   
   cdk_handle_new (&hd);
-  cdk_keydb_add_resource ("pub.gpg", 0);
-  rc = cdk_file_verify_clearsign (hd, file, "clear.txt");
+  cdk_keydb_new (&db, CDK_DBTYPE_PK_KEYRING, "pub.gpg", 7);
+  cdk_handle_set_keydb (hd, db);
+  rc = cdk_file_verify (hd, file, "clear.txt");
   if (rc)
     printf ("verify_file %s: `%s'\n", file, cdk_strerror (rc));
   cdk_handle_free (hd);
@@ -431,10 +438,11 @@ stream_clearsign (char *file)
 {
   CDK_HD hd;
   CDK_STRLIST locusr = NULL;
+  CDK_KEYDB_HD db;
   int rc;
   
   cdk_handle_new (&hd);
-  cdk_keydb_add_resource ("sec.gpg", 0);
+  cdk_keydb_new (&db, CDK_DBTYPE_SK_KEYRING, "sec.gpg", 7);
   cdk_strlist_add (&locusr, "twoaday@daredevil");
   rc = cdk_file_clearsign (hd, locusr, file, "out.asc");
   if (rc)
